import inspect
import re
import textwrap

from inflection import dasherize
from functools import partial
from functools import wraps

from typing import Any
from typing import Dict
from typing import Callable
from typing import List
from typing import Optional
from typing import Tuple
from typing import Union

from argo.workflows.client.models import (
    V1alpha1Arguments,
    V1alpha1Artifact,
    V1alpha1Inputs,
    V1alpha1Outputs,
    V1alpha1Parameter,
    V1alpha1ResourceTemplate,
    V1alpha1ScriptTemplate,
    V1alpha1Template,
    V1Container,
    V1ContainerPort,
    V1EnvFromSource,
    V1EnvVar,
    V1Lifecycle,
    V1Probe,
    V1ResourceRequirements,
    V1SecurityContext,
    V1VolumeDevice,
    V1VolumeMount,
)

from ._arguments import artifact
from ._arguments import parameter
from ._base import Prop
from ._base import Spec
from ._inputs import inputs
from ._outputs import outputs


__all__ = [
    # decorators
    "artifact",
    "closure",
    "inputs",
    "outputs",
    "parameter",
    "scope",
    "template",
    # models
    "V1alpha1Arguments",
    "V1alpha1Artifact",
    "V1alpha1Parameter",
    "V1alpha1ResourceTemplate",
    "V1alpha1ScriptTemplate",
    "V1Container",
    "V1ContainerPort",
    "V1EnvFromSource",
    "V1EnvVar",
    "V1Lifecycle",
    "V1Probe",
    "V1ResourceRequirements",
    "V1SecurityContext",
    "V1VolumeDevice",
    "V1VolumeMount",
]

# return type
T = Union[V1alpha1ResourceTemplate, V1alpha1ScriptTemplate, V1Container]


class template(Spec):

    __model__ = V1alpha1Template

    def __new__(cls, f: Callable[..., T]):
        """Workflow spec for V1alpha1Template."""
        self = super().__new__(cls, f)
        self.name = dasherize(f.__code__.co_name)

        return self


class closure(Prop, command=["python"]):
    """Workflow spec for V1alpha1Template using closure."""

    __model__ = V1alpha1ScriptTemplate

    def __init__(
        self,
        image: str,
        scope: str = None,
        *,
        env: List[V1EnvVar] = None,
        env_from: List[V1EnvFromSource] = None,
        image_pull_policy: str = None,
        lifecycle: V1Lifecycle = None,
        liveness_probe: V1Probe = None,
        ports: List[V1ContainerPort] = None,
        readiness_probe: V1Probe = None,
        resources: V1ResourceRequirements = None,
        security_context: V1SecurityContext = None,
        stdin: bool = None,
        stdin_once: bool = None,
        termination_message_path: str = None,
        termination_message_policy: str = None,
        tty: bool = None,
        volume_devices: List[V1VolumeDevice] = None,
        volume_mounts: List[V1VolumeMount] = None,
        working_dir: str = None,
    ):  # noqa: E501
        super().__init__(**self.__dict__, name="script", source="")

        self.image = image
        self.scope = scope

        if env is not None:
            self.env = env
        if env_from is not None:
            self.env_from = env_from
        if image_pull_policy is not None:
            self.image_pull_policy = image_pull_policy
        if lifecycle is not None:
            self.lifecycle = lifecycle
        if liveness_probe is not None:
            self.liveness_probe = liveness_probe
        if ports is not None:
            self.ports = ports
        if readiness_probe is not None:
            self.readiness_probe = readiness_probe
        if resources is not None:
            self.resources = resources
        if security_context is not None:
            self.security_context = security_context
        if stdin is not None:
            self.stdin = stdin
        if stdin_once is not None:
            self.stdin_once = stdin_once
        if termination_message_path is not None:
            self.termination_message_path = termination_message_path
        if termination_message_policy is not None:
            self.termination_message_policy = termination_message_policy
        if tty is not None:
            self.tty = tty
        if volume_devices is not None:
            self.volume_devices = volume_devices
        if volume_mounts is not None:
            self.volume_mounts = volume_mounts
        if working_dir is not None:
            self.working_dir = working_dir

    def __call__(self, f: Callable[..., None]) -> template:
        super().__call__(f)

        self.name = dasherize(f.__code__.co_name)

        source: List[str]
        source, _ = inspect.getsourcelines(f.__code__)

        co_start: int = 0
        for i, line in enumerate(source):
            if re.search(r"\)( -> (.+))?:[\s\n\r]+$", line):
                co_start = i + 1
                break

        self.source = textwrap.dedent("".join(source[co_start:]))

        tmpl = template(f)
        tmpl.callable = False

        tmpl.__closure__ = self.scope

        return tmpl


class scope:
    """Mark scope for closures."""

    def __init__(self, name: str):
        self.name = name

    def __call__(self, f: Callable) -> Callable:
        m = staticmethod(f)
        m.__scope__ = self.name

        return m

    @property
    def name(self) -> str:
        return self._name

    @name.setter
    def name(self, name: str):
        """Validate and set scope name.

        :raises: ValueError
        """
        valid_pattern = r"^[a-zA-Z_][a-zA-Z0-9_]*$"

        if not bool(re.match(valid_pattern, name)):
            raise ValueError(
                f"String {name} is not valid scope name."
                f"Scope name must match expression '{valid_pattern}'."
            )

        self._name = name



# coding: utf-8

"""
    Argo

    Python client for Argo Workflows  # noqa: E501

    OpenAPI spec version: master
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class V1PodSecurityContext(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'fs_group': 'str',
        'run_as_group': 'str',
        'run_as_non_root': 'bool',
        'run_as_user': 'str',
        'se_linux_options': 'V1SELinuxOptions',
        'supplemental_groups': 'list[str]',
        'sysctls': 'list[V1Sysctl]',
        'windows_options': 'V1WindowsSecurityContextOptions'
    }

    attribute_map = {
        'fs_group': 'fsGroup',
        'run_as_group': 'runAsGroup',
        'run_as_non_root': 'runAsNonRoot',
        'run_as_user': 'runAsUser',
        'se_linux_options': 'seLinuxOptions',
        'supplemental_groups': 'supplementalGroups',
        'sysctls': 'sysctls',
        'windows_options': 'windowsOptions'
    }

    def __init__(self, fs_group=None, run_as_group=None, run_as_non_root=None, run_as_user=None, se_linux_options=None, supplemental_groups=None, sysctls=None, windows_options=None):  # noqa: E501
        """V1PodSecurityContext - a model defined in Swagger"""  # noqa: E501

        self._fs_group = None
        self._run_as_group = None
        self._run_as_non_root = None
        self._run_as_user = None
        self._se_linux_options = None
        self._supplemental_groups = None
        self._sysctls = None
        self._windows_options = None
        self.discriminator = None

        if fs_group is not None:
            self.fs_group = fs_group
        if run_as_group is not None:
            self.run_as_group = run_as_group
        if run_as_non_root is not None:
            self.run_as_non_root = run_as_non_root
        if run_as_user is not None:
            self.run_as_user = run_as_user
        if se_linux_options is not None:
            self.se_linux_options = se_linux_options
        if supplemental_groups is not None:
            self.supplemental_groups = supplemental_groups
        if sysctls is not None:
            self.sysctls = sysctls
        if windows_options is not None:
            self.windows_options = windows_options

    @property
    def fs_group(self):
        """Gets the fs_group of this V1PodSecurityContext.  # noqa: E501

        1. The owning GID will be the FSGroup 2. The setgid bit is set (new files created in the volume will be owned by FSGroup) 3. The permission bits are OR'd with rw-rw----  If unset, the Kubelet will not modify the ownership and permissions of any volume. +optional  # noqa: E501

        :return: The fs_group of this V1PodSecurityContext.  # noqa: E501
        :rtype: str
        """
        return self._fs_group

    @fs_group.setter
    def fs_group(self, fs_group):
        """Sets the fs_group of this V1PodSecurityContext.

        1. The owning GID will be the FSGroup 2. The setgid bit is set (new files created in the volume will be owned by FSGroup) 3. The permission bits are OR'd with rw-rw----  If unset, the Kubelet will not modify the ownership and permissions of any volume. +optional  # noqa: E501

        :param fs_group: The fs_group of this V1PodSecurityContext.  # noqa: E501
        :type: str
        """

        self._fs_group = fs_group

    @property
    def run_as_group(self):
        """Gets the run_as_group of this V1PodSecurityContext.  # noqa: E501


        :return: The run_as_group of this V1PodSecurityContext.  # noqa: E501
        :rtype: str
        """
        return self._run_as_group

    @run_as_group.setter
    def run_as_group(self, run_as_group):
        """Sets the run_as_group of this V1PodSecurityContext.


        :param run_as_group: The run_as_group of this V1PodSecurityContext.  # noqa: E501
        :type: str
        """

        self._run_as_group = run_as_group

    @property
    def run_as_non_root(self):
        """Gets the run_as_non_root of this V1PodSecurityContext.  # noqa: E501


        :return: The run_as_non_root of this V1PodSecurityContext.  # noqa: E501
        :rtype: bool
        """
        return self._run_as_non_root

    @run_as_non_root.setter
    def run_as_non_root(self, run_as_non_root):
        """Sets the run_as_non_root of this V1PodSecurityContext.


        :param run_as_non_root: The run_as_non_root of this V1PodSecurityContext.  # noqa: E501
        :type: bool
        """

        self._run_as_non_root = run_as_non_root

    @property
    def run_as_user(self):
        """Gets the run_as_user of this V1PodSecurityContext.  # noqa: E501


        :return: The run_as_user of this V1PodSecurityContext.  # noqa: E501
        :rtype: str
        """
        return self._run_as_user

    @run_as_user.setter
    def run_as_user(self, run_as_user):
        """Sets the run_as_user of this V1PodSecurityContext.


        :param run_as_user: The run_as_user of this V1PodSecurityContext.  # noqa: E501
        :type: str
        """

        self._run_as_user = run_as_user

    @property
    def se_linux_options(self):
        """Gets the se_linux_options of this V1PodSecurityContext.  # noqa: E501


        :return: The se_linux_options of this V1PodSecurityContext.  # noqa: E501
        :rtype: V1SELinuxOptions
        """
        return self._se_linux_options

    @se_linux_options.setter
    def se_linux_options(self, se_linux_options):
        """Sets the se_linux_options of this V1PodSecurityContext.


        :param se_linux_options: The se_linux_options of this V1PodSecurityContext.  # noqa: E501
        :type: V1SELinuxOptions
        """

        self._se_linux_options = se_linux_options

    @property
    def supplemental_groups(self):
        """Gets the supplemental_groups of this V1PodSecurityContext.  # noqa: E501


        :return: The supplemental_groups of this V1PodSecurityContext.  # noqa: E501
        :rtype: list[str]
        """
        return self._supplemental_groups

    @supplemental_groups.setter
    def supplemental_groups(self, supplemental_groups):
        """Sets the supplemental_groups of this V1PodSecurityContext.


        :param supplemental_groups: The supplemental_groups of this V1PodSecurityContext.  # noqa: E501
        :type: list[str]
        """

        self._supplemental_groups = supplemental_groups

    @property
    def sysctls(self):
        """Gets the sysctls of this V1PodSecurityContext.  # noqa: E501


        :return: The sysctls of this V1PodSecurityContext.  # noqa: E501
        :rtype: list[V1Sysctl]
        """
        return self._sysctls

    @sysctls.setter
    def sysctls(self, sysctls):
        """Sets the sysctls of this V1PodSecurityContext.


        :param sysctls: The sysctls of this V1PodSecurityContext.  # noqa: E501
        :type: list[V1Sysctl]
        """

        self._sysctls = sysctls

    @property
    def windows_options(self):
        """Gets the windows_options of this V1PodSecurityContext.  # noqa: E501


        :return: The windows_options of this V1PodSecurityContext.  # noqa: E501
        :rtype: V1WindowsSecurityContextOptions
        """
        return self._windows_options

    @windows_options.setter
    def windows_options(self, windows_options):
        """Sets the windows_options of this V1PodSecurityContext.


        :param windows_options: The windows_options of this V1PodSecurityContext.  # noqa: E501
        :type: V1WindowsSecurityContextOptions
        """

        self._windows_options = windows_options

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(V1PodSecurityContext, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1PodSecurityContext):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



from inflection import dasherize
from functools import partial
from functools import wraps

from typing import Any
from typing import Dict
from typing import Callable
from typing import List
from typing import Optional
from typing import Tuple
from typing import Type
from typing import Union

from argo.workflows.client.models import (
    V1alpha1Arguments,
    V1alpha1Artifact,
    V1alpha1ContinueOn,
    V1alpha1DAGTask,
    V1alpha1Parameter,
    V1alpha1Sequence,
    V1alpha1Template,
    V1alpha1TemplateRef,
)

from ._arguments import artifact
from ._arguments import parameter
from ._base import Prop
from ._base import Spec


__all__ = [
    # decorators
    "artifact",
    "continue_on",
    "dependencies",
    "parameter",
    "task",
    "when",
    "with_items",
    "with_param",
    "with_sequence",
    # models
    "V1alpha1Template",
    "V1alpha1TemplateRef",
]

# return type
T = Union[V1alpha1Template, V1alpha1TemplateRef]


class task(Spec):

    __model__ = V1alpha1DAGTask

    def __new__(cls, f: Callable[..., T]):
        """Workflow spec for V1alpha1Template."""
        self = super().__new__(cls, f)
        self.name = dasherize(f.__code__.co_name)

        self.template: str = None
        self.template_ref: V1alpha1TemplateRef = None

        return self

    def __init_model__(self, spec: T, *args, **kwargs):
        _: Tuple[Any, ...] = args  # ignore

        if isinstance(spec, V1alpha1Template):
            self.template: str = spec.name
        elif isinstance(spec, V1alpha1TemplateRef):
            self.template_ref: V1alpha1TemplateRef = spec
        else:
            raise TypeError(f"Expected {T}, got: {type(spec)}")


class dependencies(Prop):

    __model__ = List[str]


class continue_on(Prop):

    __model__ = V1alpha1ContinueOn


class when(Prop):

    __model__ = str


class with_items(Prop):

    __model__ = List[str]


class with_param(Prop):

    __model__ = str


class with_sequence(Prop):

    __model__ = V1alpha1Sequence



from typing import Callable
from typing import List

from argo.workflows.client.models import (
    V1alpha1Outputs,
    V1alpha1Artifact,
    V1alpha1Parameter,
)

from ._base import Prop
from ._arguments import artifact as artifact
from ._arguments import parameter as parameter

__all__ = ["outputs", "V1alpha1Artifact", "V1alpha1Parameter"]


class outputs:
    """Outputs namespace."""

    class artifact(artifact, extends=("outputs", V1alpha1Outputs)):

        __model__ = V1alpha1Artifact

    class parameter(parameter, extends=("outputs", V1alpha1Outputs)):

        __model__ = V1alpha1Parameter



# coding: utf-8

"""
    Argo

    Python client for Argo Workflows  # noqa: E501

    OpenAPI spec version: master
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class V1alpha1CronWorkflowSpec(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'concurrency_policy': 'str',
        'failed_jobs_history_limit': 'int',
        'schedule': 'str',
        'starting_deadline_seconds': 'str',
        'successful_jobs_history_limit': 'int',
        'suspend': 'bool',
        'timezone': 'str',
        'workflow_spec': 'V1alpha1WorkflowSpec'
    }

    attribute_map = {
        'concurrency_policy': 'concurrencyPolicy',
        'failed_jobs_history_limit': 'failedJobsHistoryLimit',
        'schedule': 'schedule',
        'starting_deadline_seconds': 'startingDeadlineSeconds',
        'successful_jobs_history_limit': 'successfulJobsHistoryLimit',
        'suspend': 'suspend',
        'timezone': 'timezone',
        'workflow_spec': 'workflowSpec'
    }

    def __init__(self, concurrency_policy=None, failed_jobs_history_limit=None, schedule=None, starting_deadline_seconds=None, successful_jobs_history_limit=None, suspend=None, timezone=None, workflow_spec=None):  # noqa: E501
        """V1alpha1CronWorkflowSpec - a model defined in Swagger"""  # noqa: E501

        self._concurrency_policy = None
        self._failed_jobs_history_limit = None
        self._schedule = None
        self._starting_deadline_seconds = None
        self._successful_jobs_history_limit = None
        self._suspend = None
        self._timezone = None
        self._workflow_spec = None
        self.discriminator = None

        if concurrency_policy is not None:
            self.concurrency_policy = concurrency_policy
        if failed_jobs_history_limit is not None:
            self.failed_jobs_history_limit = failed_jobs_history_limit
        if schedule is not None:
            self.schedule = schedule
        if starting_deadline_seconds is not None:
            self.starting_deadline_seconds = starting_deadline_seconds
        if successful_jobs_history_limit is not None:
            self.successful_jobs_history_limit = successful_jobs_history_limit
        if suspend is not None:
            self.suspend = suspend
        if timezone is not None:
            self.timezone = timezone
        if workflow_spec is not None:
            self.workflow_spec = workflow_spec

    @property
    def concurrency_policy(self):
        """Gets the concurrency_policy of this V1alpha1CronWorkflowSpec.  # noqa: E501


        :return: The concurrency_policy of this V1alpha1CronWorkflowSpec.  # noqa: E501
        :rtype: str
        """
        return self._concurrency_policy

    @concurrency_policy.setter
    def concurrency_policy(self, concurrency_policy):
        """Sets the concurrency_policy of this V1alpha1CronWorkflowSpec.


        :param concurrency_policy: The concurrency_policy of this V1alpha1CronWorkflowSpec.  # noqa: E501
        :type: str
        """

        self._concurrency_policy = concurrency_policy

    @property
    def failed_jobs_history_limit(self):
        """Gets the failed_jobs_history_limit of this V1alpha1CronWorkflowSpec.  # noqa: E501


        :return: The failed_jobs_history_limit of this V1alpha1CronWorkflowSpec.  # noqa: E501
        :rtype: int
        """
        return self._failed_jobs_history_limit

    @failed_jobs_history_limit.setter
    def failed_jobs_history_limit(self, failed_jobs_history_limit):
        """Sets the failed_jobs_history_limit of this V1alpha1CronWorkflowSpec.


        :param failed_jobs_history_limit: The failed_jobs_history_limit of this V1alpha1CronWorkflowSpec.  # noqa: E501
        :type: int
        """

        self._failed_jobs_history_limit = failed_jobs_history_limit

    @property
    def schedule(self):
        """Gets the schedule of this V1alpha1CronWorkflowSpec.  # noqa: E501


        :return: The schedule of this V1alpha1CronWorkflowSpec.  # noqa: E501
        :rtype: str
        """
        return self._schedule

    @schedule.setter
    def schedule(self, schedule):
        """Sets the schedule of this V1alpha1CronWorkflowSpec.


        :param schedule: The schedule of this V1alpha1CronWorkflowSpec.  # noqa: E501
        :type: str
        """

        self._schedule = schedule

    @property
    def starting_deadline_seconds(self):
        """Gets the starting_deadline_seconds of this V1alpha1CronWorkflowSpec.  # noqa: E501

        StartingDeadlineSeconds is the K8s-style deadline that will limit the time a CronWorkflow will be run after its original scheduled time if it is missed.  # noqa: E501

        :return: The starting_deadline_seconds of this V1alpha1CronWorkflowSpec.  # noqa: E501
        :rtype: str
        """
        return self._starting_deadline_seconds

    @starting_deadline_seconds.setter
    def starting_deadline_seconds(self, starting_deadline_seconds):
        """Sets the starting_deadline_seconds of this V1alpha1CronWorkflowSpec.

        StartingDeadlineSeconds is the K8s-style deadline that will limit the time a CronWorkflow will be run after its original scheduled time if it is missed.  # noqa: E501

        :param starting_deadline_seconds: The starting_deadline_seconds of this V1alpha1CronWorkflowSpec.  # noqa: E501
        :type: str
        """

        self._starting_deadline_seconds = starting_deadline_seconds

    @property
    def successful_jobs_history_limit(self):
        """Gets the successful_jobs_history_limit of this V1alpha1CronWorkflowSpec.  # noqa: E501


        :return: The successful_jobs_history_limit of this V1alpha1CronWorkflowSpec.  # noqa: E501
        :rtype: int
        """
        return self._successful_jobs_history_limit

    @successful_jobs_history_limit.setter
    def successful_jobs_history_limit(self, successful_jobs_history_limit):
        """Sets the successful_jobs_history_limit of this V1alpha1CronWorkflowSpec.


        :param successful_jobs_history_limit: The successful_jobs_history_limit of this V1alpha1CronWorkflowSpec.  # noqa: E501
        :type: int
        """

        self._successful_jobs_history_limit = successful_jobs_history_limit

    @property
    def suspend(self):
        """Gets the suspend of this V1alpha1CronWorkflowSpec.  # noqa: E501


        :return: The suspend of this V1alpha1CronWorkflowSpec.  # noqa: E501
        :rtype: bool
        """
        return self._suspend

    @suspend.setter
    def suspend(self, suspend):
        """Sets the suspend of this V1alpha1CronWorkflowSpec.


        :param suspend: The suspend of this V1alpha1CronWorkflowSpec.  # noqa: E501
        :type: bool
        """

        self._suspend = suspend

    @property
    def timezone(self):
        """Gets the timezone of this V1alpha1CronWorkflowSpec.  # noqa: E501

        Timezone is the timezone against which the cron schedule will be calculated, e.g. \"Asia/Tokyo\". Default is machine's local time.  # noqa: E501

        :return: The timezone of this V1alpha1CronWorkflowSpec.  # noqa: E501
        :rtype: str
        """
        return self._timezone

    @timezone.setter
    def timezone(self, timezone):
        """Sets the timezone of this V1alpha1CronWorkflowSpec.

        Timezone is the timezone against which the cron schedule will be calculated, e.g. \"Asia/Tokyo\". Default is machine's local time.  # noqa: E501

        :param timezone: The timezone of this V1alpha1CronWorkflowSpec.  # noqa: E501
        :type: str
        """

        self._timezone = timezone

    @property
    def workflow_spec(self):
        """Gets the workflow_spec of this V1alpha1CronWorkflowSpec.  # noqa: E501


        :return: The workflow_spec of this V1alpha1CronWorkflowSpec.  # noqa: E501
        :rtype: V1alpha1WorkflowSpec
        """
        return self._workflow_spec

    @workflow_spec.setter
    def workflow_spec(self, workflow_spec):
        """Sets the workflow_spec of this V1alpha1CronWorkflowSpec.


        :param workflow_spec: The workflow_spec of this V1alpha1CronWorkflowSpec.  # noqa: E501
        :type: V1alpha1WorkflowSpec
        """

        self._workflow_spec = workflow_spec

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(V1alpha1CronWorkflowSpec, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1alpha1CronWorkflowSpec):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

# flake8: noqa

"""
    Argo

    Python client for Argo Workflows  # noqa: E501

    OpenAPI spec version: master
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

# import apis into sdk package
from argo.workflows.client.api.archived_workflow_service_api import ArchivedWorkflowServiceApi
from argo.workflows.client.api.cron_workflow_service_api import CronWorkflowServiceApi
from argo.workflows.client.api.info_service_api import InfoServiceApi
from argo.workflows.client.api.workflow_service_api import WorkflowServiceApi
from argo.workflows.client.api.workflow_template_service_api import WorkflowTemplateServiceApi
from argo.workflows.client.api.v1alpha1_api import V1alpha1Api

# import ApiClient
from argo.workflows.client.api_client import ApiClient
from argo.workflows.client.configuration import Configuration
# import models into sdk package
from argo.workflows.client.models.google_protobuf_any import GoogleProtobufAny
from argo.workflows.client.models.grpc_gateway_runtime_stream_error import GrpcGatewayRuntimeStreamError
from argo.workflows.client.models.v1_aws_elastic_block_store_volume_source import V1AWSElasticBlockStoreVolumeSource
from argo.workflows.client.models.v1_affinity import V1Affinity
from argo.workflows.client.models.v1_azure_disk_volume_source import V1AzureDiskVolumeSource
from argo.workflows.client.models.v1_azure_file_volume_source import V1AzureFileVolumeSource
from argo.workflows.client.models.v1_csi_volume_source import V1CSIVolumeSource
from argo.workflows.client.models.v1_capabilities import V1Capabilities
from argo.workflows.client.models.v1_ceph_fs_volume_source import V1CephFSVolumeSource
from argo.workflows.client.models.v1_cinder_volume_source import V1CinderVolumeSource
from argo.workflows.client.models.v1_config_map_env_source import V1ConfigMapEnvSource
from argo.workflows.client.models.v1_config_map_key_selector import V1ConfigMapKeySelector
from argo.workflows.client.models.v1_config_map_projection import V1ConfigMapProjection
from argo.workflows.client.models.v1_config_map_volume_source import V1ConfigMapVolumeSource
from argo.workflows.client.models.v1_container import V1Container
from argo.workflows.client.models.v1_container_port import V1ContainerPort
from argo.workflows.client.models.v1_create_options import V1CreateOptions
from argo.workflows.client.models.v1_delete_options import V1DeleteOptions
from argo.workflows.client.models.v1_downward_api_projection import V1DownwardAPIProjection
from argo.workflows.client.models.v1_downward_api_volume_file import V1DownwardAPIVolumeFile
from argo.workflows.client.models.v1_downward_api_volume_source import V1DownwardAPIVolumeSource
from argo.workflows.client.models.v1_empty_dir_volume_source import V1EmptyDirVolumeSource
from argo.workflows.client.models.v1_env_from_source import V1EnvFromSource
from argo.workflows.client.models.v1_env_var import V1EnvVar
from argo.workflows.client.models.v1_env_var_source import V1EnvVarSource
from argo.workflows.client.models.v1_exec_action import V1ExecAction
from argo.workflows.client.models.v1_fc_volume_source import V1FCVolumeSource
from argo.workflows.client.models.v1_fields_v1 import V1FieldsV1
from argo.workflows.client.models.v1_flex_volume_source import V1FlexVolumeSource
from argo.workflows.client.models.v1_flocker_volume_source import V1FlockerVolumeSource
from argo.workflows.client.models.v1_gce_persistent_disk_volume_source import V1GCEPersistentDiskVolumeSource
from argo.workflows.client.models.v1_get_options import V1GetOptions
from argo.workflows.client.models.v1_git_repo_volume_source import V1GitRepoVolumeSource
from argo.workflows.client.models.v1_glusterfs_volume_source import V1GlusterfsVolumeSource
from argo.workflows.client.models.v1_http_get_action import V1HTTPGetAction
from argo.workflows.client.models.v1_http_header import V1HTTPHeader
from argo.workflows.client.models.v1_handler import V1Handler
from argo.workflows.client.models.v1_host_alias import V1HostAlias
from argo.workflows.client.models.v1_host_path_volume_source import V1HostPathVolumeSource
from argo.workflows.client.models.v1_iscsi_volume_source import V1ISCSIVolumeSource
from argo.workflows.client.models.v1_int_or_string import V1IntOrString
from argo.workflows.client.models.v1_key_to_path import V1KeyToPath
from argo.workflows.client.models.v1_label_selector import V1LabelSelector
from argo.workflows.client.models.v1_label_selector_requirement import V1LabelSelectorRequirement
from argo.workflows.client.models.v1_lifecycle import V1Lifecycle
from argo.workflows.client.models.v1_list_meta import V1ListMeta
from argo.workflows.client.models.v1_list_options import V1ListOptions
from argo.workflows.client.models.v1_local_object_reference import V1LocalObjectReference
from argo.workflows.client.models.v1_managed_fields_entry import V1ManagedFieldsEntry
from argo.workflows.client.models.v1_nfs_volume_source import V1NFSVolumeSource
from argo.workflows.client.models.v1_node_affinity import V1NodeAffinity
from argo.workflows.client.models.v1_node_selector import V1NodeSelector
from argo.workflows.client.models.v1_node_selector_requirement import V1NodeSelectorRequirement
from argo.workflows.client.models.v1_node_selector_term import V1NodeSelectorTerm
from argo.workflows.client.models.v1_object_field_selector import V1ObjectFieldSelector
from argo.workflows.client.models.v1_object_meta import V1ObjectMeta
from argo.workflows.client.models.v1_object_reference import V1ObjectReference
from argo.workflows.client.models.v1_owner_reference import V1OwnerReference
from argo.workflows.client.models.v1_persistent_volume_claim import V1PersistentVolumeClaim
from argo.workflows.client.models.v1_persistent_volume_claim_condition import V1PersistentVolumeClaimCondition
from argo.workflows.client.models.v1_persistent_volume_claim_spec import V1PersistentVolumeClaimSpec
from argo.workflows.client.models.v1_persistent_volume_claim_status import V1PersistentVolumeClaimStatus
from argo.workflows.client.models.v1_persistent_volume_claim_volume_source import V1PersistentVolumeClaimVolumeSource
from argo.workflows.client.models.v1_photon_persistent_disk_volume_source import V1PhotonPersistentDiskVolumeSource
from argo.workflows.client.models.v1_pod_affinity import V1PodAffinity
from argo.workflows.client.models.v1_pod_affinity_term import V1PodAffinityTerm
from argo.workflows.client.models.v1_pod_anti_affinity import V1PodAntiAffinity
from argo.workflows.client.models.v1_pod_dns_config import V1PodDNSConfig
from argo.workflows.client.models.v1_pod_dns_config_option import V1PodDNSConfigOption
from argo.workflows.client.models.v1_pod_log_options import V1PodLogOptions
from argo.workflows.client.models.v1_pod_security_context import V1PodSecurityContext
from argo.workflows.client.models.v1_portworx_volume_source import V1PortworxVolumeSource
from argo.workflows.client.models.v1_preconditions import V1Preconditions
from argo.workflows.client.models.v1_preferred_scheduling_term import V1PreferredSchedulingTerm
from argo.workflows.client.models.v1_probe import V1Probe
from argo.workflows.client.models.v1_projected_volume_source import V1ProjectedVolumeSource
from argo.workflows.client.models.v1_quantity import V1Quantity
from argo.workflows.client.models.v1_quobyte_volume_source import V1QuobyteVolumeSource
from argo.workflows.client.models.v1_rbd_volume_source import V1RBDVolumeSource
from argo.workflows.client.models.v1_resource_field_selector import V1ResourceFieldSelector
from argo.workflows.client.models.v1_resource_requirements import V1ResourceRequirements
from argo.workflows.client.models.v1_se_linux_options import V1SELinuxOptions
from argo.workflows.client.models.v1_scale_io_volume_source import V1ScaleIOVolumeSource
from argo.workflows.client.models.v1_secret_env_source import V1SecretEnvSource
from argo.workflows.client.models.v1_secret_key_selector import V1SecretKeySelector
from argo.workflows.client.models.v1_secret_projection import V1SecretProjection
from argo.workflows.client.models.v1_secret_volume_source import V1SecretVolumeSource
from argo.workflows.client.models.v1_security_context import V1SecurityContext
from argo.workflows.client.models.v1_service_account_token_projection import V1ServiceAccountTokenProjection
from argo.workflows.client.models.v1_storage_os_volume_source import V1StorageOSVolumeSource
from argo.workflows.client.models.v1_sysctl import V1Sysctl
from argo.workflows.client.models.v1_tcp_socket_action import V1TCPSocketAction
from argo.workflows.client.models.v1_time import V1Time
from argo.workflows.client.models.v1_toleration import V1Toleration
from argo.workflows.client.models.v1_typed_local_object_reference import V1TypedLocalObjectReference
from argo.workflows.client.models.v1_volume import V1Volume
from argo.workflows.client.models.v1_volume_device import V1VolumeDevice
from argo.workflows.client.models.v1_volume_mount import V1VolumeMount
from argo.workflows.client.models.v1_volume_projection import V1VolumeProjection
from argo.workflows.client.models.v1_volume_source import V1VolumeSource
from argo.workflows.client.models.v1_vsphere_virtual_disk_volume_source import V1VsphereVirtualDiskVolumeSource
from argo.workflows.client.models.v1_weighted_pod_affinity_term import V1WeightedPodAffinityTerm
from argo.workflows.client.models.v1_windows_security_context_options import V1WindowsSecurityContextOptions
from argo.workflows.client.models.v1alpha1_archive_strategy import V1alpha1ArchiveStrategy
from argo.workflows.client.models.v1alpha1_arguments import V1alpha1Arguments
from argo.workflows.client.models.v1alpha1_artifact import V1alpha1Artifact
from argo.workflows.client.models.v1alpha1_artifact_location import V1alpha1ArtifactLocation
from argo.workflows.client.models.v1alpha1_artifact_repository_ref import V1alpha1ArtifactRepositoryRef
from argo.workflows.client.models.v1alpha1_artifactory_artifact import V1alpha1ArtifactoryArtifact
from argo.workflows.client.models.v1alpha1_artifactory_auth import V1alpha1ArtifactoryAuth
from argo.workflows.client.models.v1alpha1_backoff import V1alpha1Backoff
from argo.workflows.client.models.v1alpha1_continue_on import V1alpha1ContinueOn
from argo.workflows.client.models.v1alpha1_create_cron_workflow_request import V1alpha1CreateCronWorkflowRequest
from argo.workflows.client.models.v1alpha1_cron_workflow import V1alpha1CronWorkflow
from argo.workflows.client.models.v1alpha1_cron_workflow_list import V1alpha1CronWorkflowList
from argo.workflows.client.models.v1alpha1_cron_workflow_spec import V1alpha1CronWorkflowSpec
from argo.workflows.client.models.v1alpha1_cron_workflow_status import V1alpha1CronWorkflowStatus
from argo.workflows.client.models.v1alpha1_dag_task import V1alpha1DAGTask
from argo.workflows.client.models.v1alpha1_dag_template import V1alpha1DAGTemplate
from argo.workflows.client.models.v1alpha1_executor_config import V1alpha1ExecutorConfig
from argo.workflows.client.models.v1alpha1_git_artifact import V1alpha1GitArtifact
from argo.workflows.client.models.v1alpha1_hdfs_artifact import V1alpha1HDFSArtifact
from argo.workflows.client.models.v1alpha1_hdfs_config import V1alpha1HDFSConfig
from argo.workflows.client.models.v1alpha1_hdfs_krb_config import V1alpha1HDFSKrbConfig
from argo.workflows.client.models.v1alpha1_http_artifact import V1alpha1HTTPArtifact
from argo.workflows.client.models.v1alpha1_info_response import V1alpha1InfoResponse
from argo.workflows.client.models.v1alpha1_inputs import V1alpha1Inputs
from argo.workflows.client.models.v1alpha1_item import V1alpha1Item
from argo.workflows.client.models.v1alpha1_item_value import V1alpha1ItemValue
from argo.workflows.client.models.v1alpha1_lint_cron_workflow_request import V1alpha1LintCronWorkflowRequest
from argo.workflows.client.models.v1alpha1_log_entry import V1alpha1LogEntry
from argo.workflows.client.models.v1alpha1_metadata import V1alpha1Metadata
from argo.workflows.client.models.v1alpha1_node_status import V1alpha1NodeStatus
from argo.workflows.client.models.v1alpha1_outputs import V1alpha1Outputs
from argo.workflows.client.models.v1alpha1_parallel_steps import V1alpha1ParallelSteps
from argo.workflows.client.models.v1alpha1_parameter import V1alpha1Parameter
from argo.workflows.client.models.v1alpha1_pod_gc import V1alpha1PodGC
from argo.workflows.client.models.v1alpha1_raw_artifact import V1alpha1RawArtifact
from argo.workflows.client.models.v1alpha1_resource_template import V1alpha1ResourceTemplate
from argo.workflows.client.models.v1alpha1_retry_strategy import V1alpha1RetryStrategy
from argo.workflows.client.models.v1alpha1_s3_artifact import V1alpha1S3Artifact
from argo.workflows.client.models.v1alpha1_s3_bucket import V1alpha1S3Bucket
from argo.workflows.client.models.v1alpha1_script_template import V1alpha1ScriptTemplate
from argo.workflows.client.models.v1alpha1_sequence import V1alpha1Sequence
from argo.workflows.client.models.v1alpha1_suspend_template import V1alpha1SuspendTemplate
from argo.workflows.client.models.v1alpha1_ttl_strategy import V1alpha1TTLStrategy
from argo.workflows.client.models.v1alpha1_template import V1alpha1Template
from argo.workflows.client.models.v1alpha1_template_ref import V1alpha1TemplateRef
from argo.workflows.client.models.v1alpha1_update_cron_workflow_request import V1alpha1UpdateCronWorkflowRequest
from argo.workflows.client.models.v1alpha1_user_container import V1alpha1UserContainer
from argo.workflows.client.models.v1alpha1_value_from import V1alpha1ValueFrom
from argo.workflows.client.models.v1alpha1_workflow import V1alpha1Workflow
from argo.workflows.client.models.v1alpha1_workflow_create_request import V1alpha1WorkflowCreateRequest
from argo.workflows.client.models.v1alpha1_workflow_lint_request import V1alpha1WorkflowLintRequest
from argo.workflows.client.models.v1alpha1_workflow_list import V1alpha1WorkflowList
from argo.workflows.client.models.v1alpha1_workflow_resubmit_request import V1alpha1WorkflowResubmitRequest
from argo.workflows.client.models.v1alpha1_workflow_resume_request import V1alpha1WorkflowResumeRequest
from argo.workflows.client.models.v1alpha1_workflow_retry_request import V1alpha1WorkflowRetryRequest
from argo.workflows.client.models.v1alpha1_workflow_spec import V1alpha1WorkflowSpec
from argo.workflows.client.models.v1alpha1_workflow_status import V1alpha1WorkflowStatus
from argo.workflows.client.models.v1alpha1_workflow_step import V1alpha1WorkflowStep
from argo.workflows.client.models.v1alpha1_workflow_suspend_request import V1alpha1WorkflowSuspendRequest
from argo.workflows.client.models.v1alpha1_workflow_template import V1alpha1WorkflowTemplate
from argo.workflows.client.models.v1alpha1_workflow_template_create_request import V1alpha1WorkflowTemplateCreateRequest
from argo.workflows.client.models.v1alpha1_workflow_template_lint_request import V1alpha1WorkflowTemplateLintRequest
from argo.workflows.client.models.v1alpha1_workflow_template_list import V1alpha1WorkflowTemplateList
from argo.workflows.client.models.v1alpha1_workflow_template_spec import V1alpha1WorkflowTemplateSpec
from argo.workflows.client.models.v1alpha1_workflow_template_update_request import V1alpha1WorkflowTemplateUpdateRequest
from argo.workflows.client.models.v1alpha1_workflow_terminate_request import V1alpha1WorkflowTerminateRequest
from argo.workflows.client.models.v1alpha1_workflow_watch_event import V1alpha1WorkflowWatchEvent

from .__about__ import __version__



import re
import yaml

from typing import Any
from typing import Dict

from uuid import uuid4

try:
    from yaml import CDumper as Dumper
except ImportError:
    from yaml import Dumper

"""Argo Workflow Python DSL utilities."""

__mark = "___%s" % uuid4()


class BlockDumper(Dumper):
    def represent_scalar(self, tag, value, style=None):
        if re.search("\n", value):
            style = "|"
            # remove trailing spaces and newlines which are not allowed in YAML blocks
            value = re.sub(" +\n", "\n", value).strip()

        return super().represent_scalar(tag, value, style)


def _omitempty_rec(obj: Dict[str, Any]):
    obj[__mark] = True
    result: Dict[str, Any] = {}

    for k, v in obj.items():
        if k == __mark:
            continue

        if v is None:  # empty
            continue

        if isinstance(v, dict) and __mark not in v:
            result[k] = _omitempty_rec(v)
        elif isinstance(v, list):
            result[k] = list(
                map(lambda d: _omitempty_rec(d) if isinstance(d, dict) else d, v)
            )
        else:
            result[k] = v

    return result


def _remove_marks(obj: Dict[str, Any]):
    if __mark not in obj:
        return
    del obj[__mark]
    for v in obj.values():
        if isinstance(v, dict):
            _remove_marks(v)


def omitempty(obj: Dict[str, Any]) -> Dict[str, Any]:
    """Return copy of the object with empty values omitted."""
    try:
        result = _omitempty_rec(obj)
    finally:
        _remove_marks(obj)

    return result


def sanitize_for_serialization(obj: Dict[str, Any]) -> Dict[str, Any]:
    """Return object sanitized for serialization.

    May be used with a V1alpha1Workflow to sanitize it
    back to the original state (i.e. per manifest).
    """
    from argo.workflows.client import ApiClient

    cl = ApiClient()
    return cl.sanitize_for_serialization(obj)



"""About this package."""

__all__ = [
    "__title__",
    "__summary__",
    "__uri__",
    "__version__",
    "__author__",
    "__email__",
    "__license__",
    "__copyright__",
]

__title__ = "argo-workflows"
__summary__ = "Client for Argo Workflows"
__uri__ = "https://github.com/CermakM/argo-client-python"

__version__ = "3.0.2"

__author__ = "Marek Cermak"
__email__ = "macermak@redhat.com"

__license__ = "MIT"
__copyright__ = "Copyright 2019 {0}".format(__author__)



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from argo.workflows.client.api.archived_workflow_service_api import ArchivedWorkflowServiceApi
from argo.workflows.client.api.cron_workflow_service_api import CronWorkflowServiceApi
from argo.workflows.client.api.info_service_api import InfoServiceApi
from argo.workflows.client.api.workflow_service_api import WorkflowServiceApi
from argo.workflows.client.api.workflow_template_service_api import WorkflowTemplateServiceApi
from argo.workflows.client.api.v1alpha1_api import V1alpha1Api



# coding: utf-8

"""
    Argo

    Python client for Argo Workflows  # noqa: E501

    OpenAPI spec version: master
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class V1alpha1GitArtifact(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'depth': 'str',
        'fetch': 'list[str]',
        'insecure_ignore_host_key': 'bool',
        'password_secret': 'V1SecretKeySelector',
        'repo': 'str',
        'revision': 'str',
        'ssh_private_key_secret': 'V1SecretKeySelector',
        'username_secret': 'V1SecretKeySelector'
    }

    attribute_map = {
        'depth': 'depth',
        'fetch': 'fetch',
        'insecure_ignore_host_key': 'insecureIgnoreHostKey',
        'password_secret': 'passwordSecret',
        'repo': 'repo',
        'revision': 'revision',
        'ssh_private_key_secret': 'sshPrivateKeySecret',
        'username_secret': 'usernameSecret'
    }

    def __init__(self, depth=None, fetch=None, insecure_ignore_host_key=None, password_secret=None, repo=None, revision=None, ssh_private_key_secret=None, username_secret=None):  # noqa: E501
        """V1alpha1GitArtifact - a model defined in Swagger"""  # noqa: E501

        self._depth = None
        self._fetch = None
        self._insecure_ignore_host_key = None
        self._password_secret = None
        self._repo = None
        self._revision = None
        self._ssh_private_key_secret = None
        self._username_secret = None
        self.discriminator = None

        if depth is not None:
            self.depth = depth
        if fetch is not None:
            self.fetch = fetch
        if insecure_ignore_host_key is not None:
            self.insecure_ignore_host_key = insecure_ignore_host_key
        if password_secret is not None:
            self.password_secret = password_secret
        if repo is not None:
            self.repo = repo
        if revision is not None:
            self.revision = revision
        if ssh_private_key_secret is not None:
            self.ssh_private_key_secret = ssh_private_key_secret
        if username_secret is not None:
            self.username_secret = username_secret

    @property
    def depth(self):
        """Gets the depth of this V1alpha1GitArtifact.  # noqa: E501


        :return: The depth of this V1alpha1GitArtifact.  # noqa: E501
        :rtype: str
        """
        return self._depth

    @depth.setter
    def depth(self, depth):
        """Sets the depth of this V1alpha1GitArtifact.


        :param depth: The depth of this V1alpha1GitArtifact.  # noqa: E501
        :type: str
        """

        self._depth = depth

    @property
    def fetch(self):
        """Gets the fetch of this V1alpha1GitArtifact.  # noqa: E501


        :return: The fetch of this V1alpha1GitArtifact.  # noqa: E501
        :rtype: list[str]
        """
        return self._fetch

    @fetch.setter
    def fetch(self, fetch):
        """Sets the fetch of this V1alpha1GitArtifact.


        :param fetch: The fetch of this V1alpha1GitArtifact.  # noqa: E501
        :type: list[str]
        """

        self._fetch = fetch

    @property
    def insecure_ignore_host_key(self):
        """Gets the insecure_ignore_host_key of this V1alpha1GitArtifact.  # noqa: E501


        :return: The insecure_ignore_host_key of this V1alpha1GitArtifact.  # noqa: E501
        :rtype: bool
        """
        return self._insecure_ignore_host_key

    @insecure_ignore_host_key.setter
    def insecure_ignore_host_key(self, insecure_ignore_host_key):
        """Sets the insecure_ignore_host_key of this V1alpha1GitArtifact.


        :param insecure_ignore_host_key: The insecure_ignore_host_key of this V1alpha1GitArtifact.  # noqa: E501
        :type: bool
        """

        self._insecure_ignore_host_key = insecure_ignore_host_key

    @property
    def password_secret(self):
        """Gets the password_secret of this V1alpha1GitArtifact.  # noqa: E501


        :return: The password_secret of this V1alpha1GitArtifact.  # noqa: E501
        :rtype: V1SecretKeySelector
        """
        return self._password_secret

    @password_secret.setter
    def password_secret(self, password_secret):
        """Sets the password_secret of this V1alpha1GitArtifact.


        :param password_secret: The password_secret of this V1alpha1GitArtifact.  # noqa: E501
        :type: V1SecretKeySelector
        """

        self._password_secret = password_secret

    @property
    def repo(self):
        """Gets the repo of this V1alpha1GitArtifact.  # noqa: E501


        :return: The repo of this V1alpha1GitArtifact.  # noqa: E501
        :rtype: str
        """
        return self._repo

    @repo.setter
    def repo(self, repo):
        """Sets the repo of this V1alpha1GitArtifact.


        :param repo: The repo of this V1alpha1GitArtifact.  # noqa: E501
        :type: str
        """

        self._repo = repo

    @property
    def revision(self):
        """Gets the revision of this V1alpha1GitArtifact.  # noqa: E501


        :return: The revision of this V1alpha1GitArtifact.  # noqa: E501
        :rtype: str
        """
        return self._revision

    @revision.setter
    def revision(self, revision):
        """Sets the revision of this V1alpha1GitArtifact.


        :param revision: The revision of this V1alpha1GitArtifact.  # noqa: E501
        :type: str
        """

        self._revision = revision

    @property
    def ssh_private_key_secret(self):
        """Gets the ssh_private_key_secret of this V1alpha1GitArtifact.  # noqa: E501


        :return: The ssh_private_key_secret of this V1alpha1GitArtifact.  # noqa: E501
        :rtype: V1SecretKeySelector
        """
        return self._ssh_private_key_secret

    @ssh_private_key_secret.setter
    def ssh_private_key_secret(self, ssh_private_key_secret):
        """Sets the ssh_private_key_secret of this V1alpha1GitArtifact.


        :param ssh_private_key_secret: The ssh_private_key_secret of this V1alpha1GitArtifact.  # noqa: E501
        :type: V1SecretKeySelector
        """

        self._ssh_private_key_secret = ssh_private_key_secret

    @property
    def username_secret(self):
        """Gets the username_secret of this V1alpha1GitArtifact.  # noqa: E501


        :return: The username_secret of this V1alpha1GitArtifact.  # noqa: E501
        :rtype: V1SecretKeySelector
        """
        return self._username_secret

    @username_secret.setter
    def username_secret(self, username_secret):
        """Sets the username_secret of this V1alpha1GitArtifact.


        :param username_secret: The username_secret of this V1alpha1GitArtifact.  # noqa: E501
        :type: V1SecretKeySelector
        """

        self._username_secret = username_secret

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(V1alpha1GitArtifact, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1alpha1GitArtifact):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Argo

    Python client for Argo Workflows  # noqa: E501

    OpenAPI spec version: master
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class V1PersistentVolumeClaimSpec(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'access_modes': 'list[str]',
        'data_source': 'V1TypedLocalObjectReference',
        'resources': 'V1ResourceRequirements',
        'selector': 'V1LabelSelector',
        'storage_class_name': 'str',
        'volume_mode': 'str',
        'volume_name': 'str'
    }

    attribute_map = {
        'access_modes': 'accessModes',
        'data_source': 'dataSource',
        'resources': 'resources',
        'selector': 'selector',
        'storage_class_name': 'storageClassName',
        'volume_mode': 'volumeMode',
        'volume_name': 'volumeName'
    }

    def __init__(self, access_modes=None, data_source=None, resources=None, selector=None, storage_class_name=None, volume_mode=None, volume_name=None):  # noqa: E501
        """V1PersistentVolumeClaimSpec - a model defined in Swagger"""  # noqa: E501

        self._access_modes = None
        self._data_source = None
        self._resources = None
        self._selector = None
        self._storage_class_name = None
        self._volume_mode = None
        self._volume_name = None
        self.discriminator = None

        if access_modes is not None:
            self.access_modes = access_modes
        if data_source is not None:
            self.data_source = data_source
        if resources is not None:
            self.resources = resources
        if selector is not None:
            self.selector = selector
        if storage_class_name is not None:
            self.storage_class_name = storage_class_name
        if volume_mode is not None:
            self.volume_mode = volume_mode
        if volume_name is not None:
            self.volume_name = volume_name

    @property
    def access_modes(self):
        """Gets the access_modes of this V1PersistentVolumeClaimSpec.  # noqa: E501


        :return: The access_modes of this V1PersistentVolumeClaimSpec.  # noqa: E501
        :rtype: list[str]
        """
        return self._access_modes

    @access_modes.setter
    def access_modes(self, access_modes):
        """Sets the access_modes of this V1PersistentVolumeClaimSpec.


        :param access_modes: The access_modes of this V1PersistentVolumeClaimSpec.  # noqa: E501
        :type: list[str]
        """

        self._access_modes = access_modes

    @property
    def data_source(self):
        """Gets the data_source of this V1PersistentVolumeClaimSpec.  # noqa: E501


        :return: The data_source of this V1PersistentVolumeClaimSpec.  # noqa: E501
        :rtype: V1TypedLocalObjectReference
        """
        return self._data_source

    @data_source.setter
    def data_source(self, data_source):
        """Sets the data_source of this V1PersistentVolumeClaimSpec.


        :param data_source: The data_source of this V1PersistentVolumeClaimSpec.  # noqa: E501
        :type: V1TypedLocalObjectReference
        """

        self._data_source = data_source

    @property
    def resources(self):
        """Gets the resources of this V1PersistentVolumeClaimSpec.  # noqa: E501


        :return: The resources of this V1PersistentVolumeClaimSpec.  # noqa: E501
        :rtype: V1ResourceRequirements
        """
        return self._resources

    @resources.setter
    def resources(self, resources):
        """Sets the resources of this V1PersistentVolumeClaimSpec.


        :param resources: The resources of this V1PersistentVolumeClaimSpec.  # noqa: E501
        :type: V1ResourceRequirements
        """

        self._resources = resources

    @property
    def selector(self):
        """Gets the selector of this V1PersistentVolumeClaimSpec.  # noqa: E501


        :return: The selector of this V1PersistentVolumeClaimSpec.  # noqa: E501
        :rtype: V1LabelSelector
        """
        return self._selector

    @selector.setter
    def selector(self, selector):
        """Sets the selector of this V1PersistentVolumeClaimSpec.


        :param selector: The selector of this V1PersistentVolumeClaimSpec.  # noqa: E501
        :type: V1LabelSelector
        """

        self._selector = selector

    @property
    def storage_class_name(self):
        """Gets the storage_class_name of this V1PersistentVolumeClaimSpec.  # noqa: E501


        :return: The storage_class_name of this V1PersistentVolumeClaimSpec.  # noqa: E501
        :rtype: str
        """
        return self._storage_class_name

    @storage_class_name.setter
    def storage_class_name(self, storage_class_name):
        """Sets the storage_class_name of this V1PersistentVolumeClaimSpec.


        :param storage_class_name: The storage_class_name of this V1PersistentVolumeClaimSpec.  # noqa: E501
        :type: str
        """

        self._storage_class_name = storage_class_name

    @property
    def volume_mode(self):
        """Gets the volume_mode of this V1PersistentVolumeClaimSpec.  # noqa: E501


        :return: The volume_mode of this V1PersistentVolumeClaimSpec.  # noqa: E501
        :rtype: str
        """
        return self._volume_mode

    @volume_mode.setter
    def volume_mode(self, volume_mode):
        """Sets the volume_mode of this V1PersistentVolumeClaimSpec.


        :param volume_mode: The volume_mode of this V1PersistentVolumeClaimSpec.  # noqa: E501
        :type: str
        """

        self._volume_mode = volume_mode

    @property
    def volume_name(self):
        """Gets the volume_name of this V1PersistentVolumeClaimSpec.  # noqa: E501


        :return: The volume_name of this V1PersistentVolumeClaimSpec.  # noqa: E501
        :rtype: str
        """
        return self._volume_name

    @volume_name.setter
    def volume_name(self, volume_name):
        """Sets the volume_name of this V1PersistentVolumeClaimSpec.


        :param volume_name: The volume_name of this V1PersistentVolumeClaimSpec.  # noqa: E501
        :type: str
        """

        self._volume_name = volume_name

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(V1PersistentVolumeClaimSpec, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1PersistentVolumeClaimSpec):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



from typing import Any
from typing import Callable
from typing import List

from argo.workflows.client.models import (
    V1alpha1Arguments,
    V1alpha1Artifact,
    V1alpha1Parameter,
)

from ._base import Prop

__all__ = ["artifact", "parameter", "V1alpha1Artifact", "V1alpha1Parameter"]


class artifact(Prop, extends=("arguments", V1alpha1Arguments)):

    __model__ = V1alpha1Artifact

    def __call__(self, f: Callable):
        artifacts: List[V1alpha1Artifact] = [self]

        prop: Any
        prop_name: str
        prop_name, prop = self.__extends__

        if not hasattr(f, "__props__"):
            f.__props__ = {prop_name: prop(artifacts=artifacts)}
        else:
            arguments: Type[prop] = f.__props__.get(prop_name, prop())

            if not getattr(arguments, "artifacts"):
                arguments.artifacts = artifacts
            else:
                arguments.artifacts.extend(artifacts)

            f.__props__[prop_name] = arguments

        return f

class parameter(Prop, extends=("arguments", V1alpha1Arguments)):

    __model__ = V1alpha1Parameter

    def __call__(self, f: Callable):
        parameters: List[V1alpha1Parameter] = [self]

        prop: Any
        prop_name: str
        prop_name, prop = self.__extends__

        if not hasattr(f, "__props__"):
            f.__props__ = {prop_name: prop(parameters=parameters)}
        else:
            arguments: Type[prop] = f.__props__.get(prop_name, prop())

            if not getattr(arguments, "parameters"):
                arguments.parameters = parameters
            else:
                arguments.parameters.extend(parameters)

            f.__props__[prop_name] = arguments

        return f



# coding: utf-8

"""
    Argo

    Python client for Argo Workflows  # noqa: E501

    OpenAPI spec version: master
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class V1ConfigMapEnvSource(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'local_object_reference': 'V1LocalObjectReference',
        'optional': 'bool'
    }

    attribute_map = {
        'local_object_reference': 'localObjectReference',
        'optional': 'optional'
    }

    def __init__(self, local_object_reference=None, optional=None):  # noqa: E501
        """V1ConfigMapEnvSource - a model defined in Swagger"""  # noqa: E501

        self._local_object_reference = None
        self._optional = None
        self.discriminator = None

        if local_object_reference is not None:
            self.local_object_reference = local_object_reference
        if optional is not None:
            self.optional = optional

    @property
    def local_object_reference(self):
        """Gets the local_object_reference of this V1ConfigMapEnvSource.  # noqa: E501

        The ConfigMap to select from.  # noqa: E501

        :return: The local_object_reference of this V1ConfigMapEnvSource.  # noqa: E501
        :rtype: V1LocalObjectReference
        """
        return self._local_object_reference

    @local_object_reference.setter
    def local_object_reference(self, local_object_reference):
        """Sets the local_object_reference of this V1ConfigMapEnvSource.

        The ConfigMap to select from.  # noqa: E501

        :param local_object_reference: The local_object_reference of this V1ConfigMapEnvSource.  # noqa: E501
        :type: V1LocalObjectReference
        """

        self._local_object_reference = local_object_reference

    @property
    def optional(self):
        """Gets the optional of this V1ConfigMapEnvSource.  # noqa: E501


        :return: The optional of this V1ConfigMapEnvSource.  # noqa: E501
        :rtype: bool
        """
        return self._optional

    @optional.setter
    def optional(self, optional):
        """Sets the optional of this V1ConfigMapEnvSource.


        :param optional: The optional of this V1ConfigMapEnvSource.  # noqa: E501
        :type: bool
        """

        self._optional = optional

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(V1ConfigMapEnvSource, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1ConfigMapEnvSource):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Argo

    Python client for Argo Workflows  # noqa: E501

    OpenAPI spec version: master
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class V1alpha1NodeStatus(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'boundary_id': 'str',
        'children': 'list[str]',
        'daemoned': 'bool',
        'display_name': 'str',
        'finished_at': 'V1Time',
        'id': 'str',
        'inputs': 'V1alpha1Inputs',
        'message': 'str',
        'name': 'str',
        'outbound_nodes': 'list[str]',
        'outputs': 'V1alpha1Outputs',
        'phase': 'str',
        'pod_ip': 'str',
        'started_at': 'V1Time',
        'stored_template_id': 'str',
        'template_name': 'str',
        'template_ref': 'V1alpha1TemplateRef',
        'template_scope': 'str',
        'type': 'str',
        'workflow_template_name': 'str'
    }

    attribute_map = {
        'boundary_id': 'boundaryID',
        'children': 'children',
        'daemoned': 'daemoned',
        'display_name': 'displayName',
        'finished_at': 'finishedAt',
        'id': 'id',
        'inputs': 'inputs',
        'message': 'message',
        'name': 'name',
        'outbound_nodes': 'outboundNodes',
        'outputs': 'outputs',
        'phase': 'phase',
        'pod_ip': 'podIP',
        'started_at': 'startedAt',
        'stored_template_id': 'storedTemplateID',
        'template_name': 'templateName',
        'template_ref': 'templateRef',
        'template_scope': 'templateScope',
        'type': 'type',
        'workflow_template_name': 'workflowTemplateName'
    }

    def __init__(self, boundary_id=None, children=None, daemoned=None, display_name=None, finished_at=None, id=None, inputs=None, message=None, name=None, outbound_nodes=None, outputs=None, phase=None, pod_ip=None, started_at=None, stored_template_id=None, template_name=None, template_ref=None, template_scope=None, type=None, workflow_template_name=None):  # noqa: E501
        """V1alpha1NodeStatus - a model defined in Swagger"""  # noqa: E501

        self._boundary_id = None
        self._children = None
        self._daemoned = None
        self._display_name = None
        self._finished_at = None
        self._id = None
        self._inputs = None
        self._message = None
        self._name = None
        self._outbound_nodes = None
        self._outputs = None
        self._phase = None
        self._pod_ip = None
        self._started_at = None
        self._stored_template_id = None
        self._template_name = None
        self._template_ref = None
        self._template_scope = None
        self._type = None
        self._workflow_template_name = None
        self.discriminator = None

        if boundary_id is not None:
            self.boundary_id = boundary_id
        if children is not None:
            self.children = children
        if daemoned is not None:
            self.daemoned = daemoned
        if display_name is not None:
            self.display_name = display_name
        if finished_at is not None:
            self.finished_at = finished_at
        if id is not None:
            self.id = id
        if inputs is not None:
            self.inputs = inputs
        if message is not None:
            self.message = message
        if name is not None:
            self.name = name
        if outbound_nodes is not None:
            self.outbound_nodes = outbound_nodes
        if outputs is not None:
            self.outputs = outputs
        if phase is not None:
            self.phase = phase
        if pod_ip is not None:
            self.pod_ip = pod_ip
        if started_at is not None:
            self.started_at = started_at
        if stored_template_id is not None:
            self.stored_template_id = stored_template_id
        if template_name is not None:
            self.template_name = template_name
        if template_ref is not None:
            self.template_ref = template_ref
        if template_scope is not None:
            self.template_scope = template_scope
        if type is not None:
            self.type = type
        if workflow_template_name is not None:
            self.workflow_template_name = workflow_template_name

    @property
    def boundary_id(self):
        """Gets the boundary_id of this V1alpha1NodeStatus.  # noqa: E501


        :return: The boundary_id of this V1alpha1NodeStatus.  # noqa: E501
        :rtype: str
        """
        return self._boundary_id

    @boundary_id.setter
    def boundary_id(self, boundary_id):
        """Sets the boundary_id of this V1alpha1NodeStatus.


        :param boundary_id: The boundary_id of this V1alpha1NodeStatus.  # noqa: E501
        :type: str
        """

        self._boundary_id = boundary_id

    @property
    def children(self):
        """Gets the children of this V1alpha1NodeStatus.  # noqa: E501


        :return: The children of this V1alpha1NodeStatus.  # noqa: E501
        :rtype: list[str]
        """
        return self._children

    @children.setter
    def children(self, children):
        """Sets the children of this V1alpha1NodeStatus.


        :param children: The children of this V1alpha1NodeStatus.  # noqa: E501
        :type: list[str]
        """

        self._children = children

    @property
    def daemoned(self):
        """Gets the daemoned of this V1alpha1NodeStatus.  # noqa: E501


        :return: The daemoned of this V1alpha1NodeStatus.  # noqa: E501
        :rtype: bool
        """
        return self._daemoned

    @daemoned.setter
    def daemoned(self, daemoned):
        """Sets the daemoned of this V1alpha1NodeStatus.


        :param daemoned: The daemoned of this V1alpha1NodeStatus.  # noqa: E501
        :type: bool
        """

        self._daemoned = daemoned

    @property
    def display_name(self):
        """Gets the display_name of this V1alpha1NodeStatus.  # noqa: E501


        :return: The display_name of this V1alpha1NodeStatus.  # noqa: E501
        :rtype: str
        """
        return self._display_name

    @display_name.setter
    def display_name(self, display_name):
        """Sets the display_name of this V1alpha1NodeStatus.


        :param display_name: The display_name of this V1alpha1NodeStatus.  # noqa: E501
        :type: str
        """

        self._display_name = display_name

    @property
    def finished_at(self):
        """Gets the finished_at of this V1alpha1NodeStatus.  # noqa: E501


        :return: The finished_at of this V1alpha1NodeStatus.  # noqa: E501
        :rtype: V1Time
        """
        return self._finished_at

    @finished_at.setter
    def finished_at(self, finished_at):
        """Sets the finished_at of this V1alpha1NodeStatus.


        :param finished_at: The finished_at of this V1alpha1NodeStatus.  # noqa: E501
        :type: V1Time
        """

        self._finished_at = finished_at

    @property
    def id(self):
        """Gets the id of this V1alpha1NodeStatus.  # noqa: E501


        :return: The id of this V1alpha1NodeStatus.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this V1alpha1NodeStatus.


        :param id: The id of this V1alpha1NodeStatus.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def inputs(self):
        """Gets the inputs of this V1alpha1NodeStatus.  # noqa: E501


        :return: The inputs of this V1alpha1NodeStatus.  # noqa: E501
        :rtype: V1alpha1Inputs
        """
        return self._inputs

    @inputs.setter
    def inputs(self, inputs):
        """Sets the inputs of this V1alpha1NodeStatus.


        :param inputs: The inputs of this V1alpha1NodeStatus.  # noqa: E501
        :type: V1alpha1Inputs
        """

        self._inputs = inputs

    @property
    def message(self):
        """Gets the message of this V1alpha1NodeStatus.  # noqa: E501

        A human readable message indicating details about why the node is in this condition.  # noqa: E501

        :return: The message of this V1alpha1NodeStatus.  # noqa: E501
        :rtype: str
        """
        return self._message

    @message.setter
    def message(self, message):
        """Sets the message of this V1alpha1NodeStatus.

        A human readable message indicating details about why the node is in this condition.  # noqa: E501

        :param message: The message of this V1alpha1NodeStatus.  # noqa: E501
        :type: str
        """

        self._message = message

    @property
    def name(self):
        """Gets the name of this V1alpha1NodeStatus.  # noqa: E501


        :return: The name of this V1alpha1NodeStatus.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this V1alpha1NodeStatus.


        :param name: The name of this V1alpha1NodeStatus.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def outbound_nodes(self):
        """Gets the outbound_nodes of this V1alpha1NodeStatus.  # noqa: E501

        OutboundNodes tracks the node IDs which are considered \"outbound\" nodes to a template invocation. For every invocation of a template, there are nodes which we considered as \"outbound\". Essentially, these are last nodes in the execution sequence to run, before the template is considered completed. These nodes are then connected as parents to a following step.  In the case of single pod steps (i.e. container, script, resource templates), this list will be nil since the pod itself is already considered the \"outbound\" node. In the case of DAGs, outbound nodes are the \"target\" tasks (tasks with no children). In the case of steps, outbound nodes are all the containers involved in the last step group. NOTE: since templates are composable, the list of outbound nodes are carried upwards when a DAG/steps template invokes another DAG/steps template. In other words, the outbound nodes of a template, will be a superset of the outbound nodes of its last children.  # noqa: E501

        :return: The outbound_nodes of this V1alpha1NodeStatus.  # noqa: E501
        :rtype: list[str]
        """
        return self._outbound_nodes

    @outbound_nodes.setter
    def outbound_nodes(self, outbound_nodes):
        """Sets the outbound_nodes of this V1alpha1NodeStatus.

        OutboundNodes tracks the node IDs which are considered \"outbound\" nodes to a template invocation. For every invocation of a template, there are nodes which we considered as \"outbound\". Essentially, these are last nodes in the execution sequence to run, before the template is considered completed. These nodes are then connected as parents to a following step.  In the case of single pod steps (i.e. container, script, resource templates), this list will be nil since the pod itself is already considered the \"outbound\" node. In the case of DAGs, outbound nodes are the \"target\" tasks (tasks with no children). In the case of steps, outbound nodes are all the containers involved in the last step group. NOTE: since templates are composable, the list of outbound nodes are carried upwards when a DAG/steps template invokes another DAG/steps template. In other words, the outbound nodes of a template, will be a superset of the outbound nodes of its last children.  # noqa: E501

        :param outbound_nodes: The outbound_nodes of this V1alpha1NodeStatus.  # noqa: E501
        :type: list[str]
        """

        self._outbound_nodes = outbound_nodes

    @property
    def outputs(self):
        """Gets the outputs of this V1alpha1NodeStatus.  # noqa: E501


        :return: The outputs of this V1alpha1NodeStatus.  # noqa: E501
        :rtype: V1alpha1Outputs
        """
        return self._outputs

    @outputs.setter
    def outputs(self, outputs):
        """Sets the outputs of this V1alpha1NodeStatus.


        :param outputs: The outputs of this V1alpha1NodeStatus.  # noqa: E501
        :type: V1alpha1Outputs
        """

        self._outputs = outputs

    @property
    def phase(self):
        """Gets the phase of this V1alpha1NodeStatus.  # noqa: E501

        Phase a simple, high-level summary of where the node is in its lifecycle. Can be used as a state machine.  # noqa: E501

        :return: The phase of this V1alpha1NodeStatus.  # noqa: E501
        :rtype: str
        """
        return self._phase

    @phase.setter
    def phase(self, phase):
        """Sets the phase of this V1alpha1NodeStatus.

        Phase a simple, high-level summary of where the node is in its lifecycle. Can be used as a state machine.  # noqa: E501

        :param phase: The phase of this V1alpha1NodeStatus.  # noqa: E501
        :type: str
        """

        self._phase = phase

    @property
    def pod_ip(self):
        """Gets the pod_ip of this V1alpha1NodeStatus.  # noqa: E501


        :return: The pod_ip of this V1alpha1NodeStatus.  # noqa: E501
        :rtype: str
        """
        return self._pod_ip

    @pod_ip.setter
    def pod_ip(self, pod_ip):
        """Sets the pod_ip of this V1alpha1NodeStatus.


        :param pod_ip: The pod_ip of this V1alpha1NodeStatus.  # noqa: E501
        :type: str
        """

        self._pod_ip = pod_ip

    @property
    def started_at(self):
        """Gets the started_at of this V1alpha1NodeStatus.  # noqa: E501


        :return: The started_at of this V1alpha1NodeStatus.  # noqa: E501
        :rtype: V1Time
        """
        return self._started_at

    @started_at.setter
    def started_at(self, started_at):
        """Sets the started_at of this V1alpha1NodeStatus.


        :param started_at: The started_at of this V1alpha1NodeStatus.  # noqa: E501
        :type: V1Time
        """

        self._started_at = started_at

    @property
    def stored_template_id(self):
        """Gets the stored_template_id of this V1alpha1NodeStatus.  # noqa: E501

        StoredTemplateID is the ID of stored template. DEPRECATED: This value is not used anymore.  # noqa: E501

        :return: The stored_template_id of this V1alpha1NodeStatus.  # noqa: E501
        :rtype: str
        """
        return self._stored_template_id

    @stored_template_id.setter
    def stored_template_id(self, stored_template_id):
        """Sets the stored_template_id of this V1alpha1NodeStatus.

        StoredTemplateID is the ID of stored template. DEPRECATED: This value is not used anymore.  # noqa: E501

        :param stored_template_id: The stored_template_id of this V1alpha1NodeStatus.  # noqa: E501
        :type: str
        """

        self._stored_template_id = stored_template_id

    @property
    def template_name(self):
        """Gets the template_name of this V1alpha1NodeStatus.  # noqa: E501


        :return: The template_name of this V1alpha1NodeStatus.  # noqa: E501
        :rtype: str
        """
        return self._template_name

    @template_name.setter
    def template_name(self, template_name):
        """Sets the template_name of this V1alpha1NodeStatus.


        :param template_name: The template_name of this V1alpha1NodeStatus.  # noqa: E501
        :type: str
        """

        self._template_name = template_name

    @property
    def template_ref(self):
        """Gets the template_ref of this V1alpha1NodeStatus.  # noqa: E501


        :return: The template_ref of this V1alpha1NodeStatus.  # noqa: E501
        :rtype: V1alpha1TemplateRef
        """
        return self._template_ref

    @template_ref.setter
    def template_ref(self, template_ref):
        """Sets the template_ref of this V1alpha1NodeStatus.


        :param template_ref: The template_ref of this V1alpha1NodeStatus.  # noqa: E501
        :type: V1alpha1TemplateRef
        """

        self._template_ref = template_ref

    @property
    def template_scope(self):
        """Gets the template_scope of this V1alpha1NodeStatus.  # noqa: E501

        TemplateScope is the template scope in which the template of this node was retrieved.  # noqa: E501

        :return: The template_scope of this V1alpha1NodeStatus.  # noqa: E501
        :rtype: str
        """
        return self._template_scope

    @template_scope.setter
    def template_scope(self, template_scope):
        """Sets the template_scope of this V1alpha1NodeStatus.

        TemplateScope is the template scope in which the template of this node was retrieved.  # noqa: E501

        :param template_scope: The template_scope of this V1alpha1NodeStatus.  # noqa: E501
        :type: str
        """

        self._template_scope = template_scope

    @property
    def type(self):
        """Gets the type of this V1alpha1NodeStatus.  # noqa: E501


        :return: The type of this V1alpha1NodeStatus.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this V1alpha1NodeStatus.


        :param type: The type of this V1alpha1NodeStatus.  # noqa: E501
        :type: str
        """

        self._type = type

    @property
    def workflow_template_name(self):
        """Gets the workflow_template_name of this V1alpha1NodeStatus.  # noqa: E501

        WorkflowTemplateName is the WorkflowTemplate resource name on which the resolved template of this node is retrieved. DEPRECATED: This value is not used anymore.  # noqa: E501

        :return: The workflow_template_name of this V1alpha1NodeStatus.  # noqa: E501
        :rtype: str
        """
        return self._workflow_template_name

    @workflow_template_name.setter
    def workflow_template_name(self, workflow_template_name):
        """Sets the workflow_template_name of this V1alpha1NodeStatus.

        WorkflowTemplateName is the WorkflowTemplate resource name on which the resolved template of this node is retrieved. DEPRECATED: This value is not used anymore.  # noqa: E501

        :param workflow_template_name: The workflow_template_name of this V1alpha1NodeStatus.  # noqa: E501
        :type: str
        """

        self._workflow_template_name = workflow_template_name

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(V1alpha1NodeStatus, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1alpha1NodeStatus):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Argo

    Python client for Argo Workflows  # noqa: E501

    OpenAPI spec version: master
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class V1Container(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'args': 'list[str]',
        'command': 'list[str]',
        'env': 'list[V1EnvVar]',
        'env_from': 'list[V1EnvFromSource]',
        'image': 'str',
        'image_pull_policy': 'str',
        'lifecycle': 'V1Lifecycle',
        'liveness_probe': 'V1Probe',
        'name': 'str',
        'ports': 'list[V1ContainerPort]',
        'readiness_probe': 'V1Probe',
        'resources': 'V1ResourceRequirements',
        'security_context': 'V1SecurityContext',
        'startup_probe': 'V1Probe',
        'stdin': 'bool',
        'stdin_once': 'bool',
        'termination_message_path': 'str',
        'termination_message_policy': 'str',
        'tty': 'bool',
        'volume_devices': 'list[V1VolumeDevice]',
        'volume_mounts': 'list[V1VolumeMount]',
        'working_dir': 'str'
    }

    attribute_map = {
        'args': 'args',
        'command': 'command',
        'env': 'env',
        'env_from': 'envFrom',
        'image': 'image',
        'image_pull_policy': 'imagePullPolicy',
        'lifecycle': 'lifecycle',
        'liveness_probe': 'livenessProbe',
        'name': 'name',
        'ports': 'ports',
        'readiness_probe': 'readinessProbe',
        'resources': 'resources',
        'security_context': 'securityContext',
        'startup_probe': 'startupProbe',
        'stdin': 'stdin',
        'stdin_once': 'stdinOnce',
        'termination_message_path': 'terminationMessagePath',
        'termination_message_policy': 'terminationMessagePolicy',
        'tty': 'tty',
        'volume_devices': 'volumeDevices',
        'volume_mounts': 'volumeMounts',
        'working_dir': 'workingDir'
    }

    def __init__(self, args=None, command=None, env=None, env_from=None, image=None, image_pull_policy=None, lifecycle=None, liveness_probe=None, name=None, ports=None, readiness_probe=None, resources=None, security_context=None, startup_probe=None, stdin=None, stdin_once=None, termination_message_path=None, termination_message_policy=None, tty=None, volume_devices=None, volume_mounts=None, working_dir=None):  # noqa: E501
        """V1Container - a model defined in Swagger"""  # noqa: E501

        self._args = None
        self._command = None
        self._env = None
        self._env_from = None
        self._image = None
        self._image_pull_policy = None
        self._lifecycle = None
        self._liveness_probe = None
        self._name = None
        self._ports = None
        self._readiness_probe = None
        self._resources = None
        self._security_context = None
        self._startup_probe = None
        self._stdin = None
        self._stdin_once = None
        self._termination_message_path = None
        self._termination_message_policy = None
        self._tty = None
        self._volume_devices = None
        self._volume_mounts = None
        self._working_dir = None
        self.discriminator = None

        if args is not None:
            self.args = args
        if command is not None:
            self.command = command
        if env is not None:
            self.env = env
        if env_from is not None:
            self.env_from = env_from
        if image is not None:
            self.image = image
        if image_pull_policy is not None:
            self.image_pull_policy = image_pull_policy
        if lifecycle is not None:
            self.lifecycle = lifecycle
        if liveness_probe is not None:
            self.liveness_probe = liveness_probe
        if name is not None:
            self.name = name
        if ports is not None:
            self.ports = ports
        if readiness_probe is not None:
            self.readiness_probe = readiness_probe
        if resources is not None:
            self.resources = resources
        if security_context is not None:
            self.security_context = security_context
        if startup_probe is not None:
            self.startup_probe = startup_probe
        if stdin is not None:
            self.stdin = stdin
        if stdin_once is not None:
            self.stdin_once = stdin_once
        if termination_message_path is not None:
            self.termination_message_path = termination_message_path
        if termination_message_policy is not None:
            self.termination_message_policy = termination_message_policy
        if tty is not None:
            self.tty = tty
        if volume_devices is not None:
            self.volume_devices = volume_devices
        if volume_mounts is not None:
            self.volume_mounts = volume_mounts
        if working_dir is not None:
            self.working_dir = working_dir

    @property
    def args(self):
        """Gets the args of this V1Container.  # noqa: E501


        :return: The args of this V1Container.  # noqa: E501
        :rtype: list[str]
        """
        return self._args

    @args.setter
    def args(self, args):
        """Sets the args of this V1Container.


        :param args: The args of this V1Container.  # noqa: E501
        :type: list[str]
        """

        self._args = args

    @property
    def command(self):
        """Gets the command of this V1Container.  # noqa: E501


        :return: The command of this V1Container.  # noqa: E501
        :rtype: list[str]
        """
        return self._command

    @command.setter
    def command(self, command):
        """Sets the command of this V1Container.


        :param command: The command of this V1Container.  # noqa: E501
        :type: list[str]
        """

        self._command = command

    @property
    def env(self):
        """Gets the env of this V1Container.  # noqa: E501


        :return: The env of this V1Container.  # noqa: E501
        :rtype: list[V1EnvVar]
        """
        return self._env

    @env.setter
    def env(self, env):
        """Sets the env of this V1Container.


        :param env: The env of this V1Container.  # noqa: E501
        :type: list[V1EnvVar]
        """

        self._env = env

    @property
    def env_from(self):
        """Gets the env_from of this V1Container.  # noqa: E501


        :return: The env_from of this V1Container.  # noqa: E501
        :rtype: list[V1EnvFromSource]
        """
        return self._env_from

    @env_from.setter
    def env_from(self, env_from):
        """Sets the env_from of this V1Container.


        :param env_from: The env_from of this V1Container.  # noqa: E501
        :type: list[V1EnvFromSource]
        """

        self._env_from = env_from

    @property
    def image(self):
        """Gets the image of this V1Container.  # noqa: E501


        :return: The image of this V1Container.  # noqa: E501
        :rtype: str
        """
        return self._image

    @image.setter
    def image(self, image):
        """Sets the image of this V1Container.


        :param image: The image of this V1Container.  # noqa: E501
        :type: str
        """

        self._image = image

    @property
    def image_pull_policy(self):
        """Gets the image_pull_policy of this V1Container.  # noqa: E501


        :return: The image_pull_policy of this V1Container.  # noqa: E501
        :rtype: str
        """
        return self._image_pull_policy

    @image_pull_policy.setter
    def image_pull_policy(self, image_pull_policy):
        """Sets the image_pull_policy of this V1Container.


        :param image_pull_policy: The image_pull_policy of this V1Container.  # noqa: E501
        :type: str
        """

        self._image_pull_policy = image_pull_policy

    @property
    def lifecycle(self):
        """Gets the lifecycle of this V1Container.  # noqa: E501


        :return: The lifecycle of this V1Container.  # noqa: E501
        :rtype: V1Lifecycle
        """
        return self._lifecycle

    @lifecycle.setter
    def lifecycle(self, lifecycle):
        """Sets the lifecycle of this V1Container.


        :param lifecycle: The lifecycle of this V1Container.  # noqa: E501
        :type: V1Lifecycle
        """

        self._lifecycle = lifecycle

    @property
    def liveness_probe(self):
        """Gets the liveness_probe of this V1Container.  # noqa: E501


        :return: The liveness_probe of this V1Container.  # noqa: E501
        :rtype: V1Probe
        """
        return self._liveness_probe

    @liveness_probe.setter
    def liveness_probe(self, liveness_probe):
        """Sets the liveness_probe of this V1Container.


        :param liveness_probe: The liveness_probe of this V1Container.  # noqa: E501
        :type: V1Probe
        """

        self._liveness_probe = liveness_probe

    @property
    def name(self):
        """Gets the name of this V1Container.  # noqa: E501

        Name of the container specified as a DNS_LABEL. Each container in a pod must have a unique name (DNS_LABEL). Cannot be updated.  # noqa: E501

        :return: The name of this V1Container.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this V1Container.

        Name of the container specified as a DNS_LABEL. Each container in a pod must have a unique name (DNS_LABEL). Cannot be updated.  # noqa: E501

        :param name: The name of this V1Container.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def ports(self):
        """Gets the ports of this V1Container.  # noqa: E501


        :return: The ports of this V1Container.  # noqa: E501
        :rtype: list[V1ContainerPort]
        """
        return self._ports

    @ports.setter
    def ports(self, ports):
        """Sets the ports of this V1Container.


        :param ports: The ports of this V1Container.  # noqa: E501
        :type: list[V1ContainerPort]
        """

        self._ports = ports

    @property
    def readiness_probe(self):
        """Gets the readiness_probe of this V1Container.  # noqa: E501


        :return: The readiness_probe of this V1Container.  # noqa: E501
        :rtype: V1Probe
        """
        return self._readiness_probe

    @readiness_probe.setter
    def readiness_probe(self, readiness_probe):
        """Sets the readiness_probe of this V1Container.


        :param readiness_probe: The readiness_probe of this V1Container.  # noqa: E501
        :type: V1Probe
        """

        self._readiness_probe = readiness_probe

    @property
    def resources(self):
        """Gets the resources of this V1Container.  # noqa: E501


        :return: The resources of this V1Container.  # noqa: E501
        :rtype: V1ResourceRequirements
        """
        return self._resources

    @resources.setter
    def resources(self, resources):
        """Sets the resources of this V1Container.


        :param resources: The resources of this V1Container.  # noqa: E501
        :type: V1ResourceRequirements
        """

        self._resources = resources

    @property
    def security_context(self):
        """Gets the security_context of this V1Container.  # noqa: E501


        :return: The security_context of this V1Container.  # noqa: E501
        :rtype: V1SecurityContext
        """
        return self._security_context

    @security_context.setter
    def security_context(self, security_context):
        """Sets the security_context of this V1Container.


        :param security_context: The security_context of this V1Container.  # noqa: E501
        :type: V1SecurityContext
        """

        self._security_context = security_context

    @property
    def startup_probe(self):
        """Gets the startup_probe of this V1Container.  # noqa: E501


        :return: The startup_probe of this V1Container.  # noqa: E501
        :rtype: V1Probe
        """
        return self._startup_probe

    @startup_probe.setter
    def startup_probe(self, startup_probe):
        """Sets the startup_probe of this V1Container.


        :param startup_probe: The startup_probe of this V1Container.  # noqa: E501
        :type: V1Probe
        """

        self._startup_probe = startup_probe

    @property
    def stdin(self):
        """Gets the stdin of this V1Container.  # noqa: E501


        :return: The stdin of this V1Container.  # noqa: E501
        :rtype: bool
        """
        return self._stdin

    @stdin.setter
    def stdin(self, stdin):
        """Sets the stdin of this V1Container.


        :param stdin: The stdin of this V1Container.  # noqa: E501
        :type: bool
        """

        self._stdin = stdin

    @property
    def stdin_once(self):
        """Gets the stdin_once of this V1Container.  # noqa: E501


        :return: The stdin_once of this V1Container.  # noqa: E501
        :rtype: bool
        """
        return self._stdin_once

    @stdin_once.setter
    def stdin_once(self, stdin_once):
        """Sets the stdin_once of this V1Container.


        :param stdin_once: The stdin_once of this V1Container.  # noqa: E501
        :type: bool
        """

        self._stdin_once = stdin_once

    @property
    def termination_message_path(self):
        """Gets the termination_message_path of this V1Container.  # noqa: E501


        :return: The termination_message_path of this V1Container.  # noqa: E501
        :rtype: str
        """
        return self._termination_message_path

    @termination_message_path.setter
    def termination_message_path(self, termination_message_path):
        """Sets the termination_message_path of this V1Container.


        :param termination_message_path: The termination_message_path of this V1Container.  # noqa: E501
        :type: str
        """

        self._termination_message_path = termination_message_path

    @property
    def termination_message_policy(self):
        """Gets the termination_message_policy of this V1Container.  # noqa: E501


        :return: The termination_message_policy of this V1Container.  # noqa: E501
        :rtype: str
        """
        return self._termination_message_policy

    @termination_message_policy.setter
    def termination_message_policy(self, termination_message_policy):
        """Sets the termination_message_policy of this V1Container.


        :param termination_message_policy: The termination_message_policy of this V1Container.  # noqa: E501
        :type: str
        """

        self._termination_message_policy = termination_message_policy

    @property
    def tty(self):
        """Gets the tty of this V1Container.  # noqa: E501


        :return: The tty of this V1Container.  # noqa: E501
        :rtype: bool
        """
        return self._tty

    @tty.setter
    def tty(self, tty):
        """Sets the tty of this V1Container.


        :param tty: The tty of this V1Container.  # noqa: E501
        :type: bool
        """

        self._tty = tty

    @property
    def volume_devices(self):
        """Gets the volume_devices of this V1Container.  # noqa: E501


        :return: The volume_devices of this V1Container.  # noqa: E501
        :rtype: list[V1VolumeDevice]
        """
        return self._volume_devices

    @volume_devices.setter
    def volume_devices(self, volume_devices):
        """Sets the volume_devices of this V1Container.


        :param volume_devices: The volume_devices of this V1Container.  # noqa: E501
        :type: list[V1VolumeDevice]
        """

        self._volume_devices = volume_devices

    @property
    def volume_mounts(self):
        """Gets the volume_mounts of this V1Container.  # noqa: E501


        :return: The volume_mounts of this V1Container.  # noqa: E501
        :rtype: list[V1VolumeMount]
        """
        return self._volume_mounts

    @volume_mounts.setter
    def volume_mounts(self, volume_mounts):
        """Sets the volume_mounts of this V1Container.


        :param volume_mounts: The volume_mounts of this V1Container.  # noqa: E501
        :type: list[V1VolumeMount]
        """

        self._volume_mounts = volume_mounts

    @property
    def working_dir(self):
        """Gets the working_dir of this V1Container.  # noqa: E501


        :return: The working_dir of this V1Container.  # noqa: E501
        :rtype: str
        """
        return self._working_dir

    @working_dir.setter
    def working_dir(self, working_dir):
        """Sets the working_dir of this V1Container.


        :param working_dir: The working_dir of this V1Container.  # noqa: E501
        :type: str
        """

        self._working_dir = working_dir

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(V1Container, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1Container):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Argo

    Python client for Argo Workflows  # noqa: E501

    OpenAPI spec version: master
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class V1alpha1ArtifactLocation(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'archive_logs': 'bool',
        'artifactory': 'V1alpha1ArtifactoryArtifact',
        'git': 'V1alpha1GitArtifact',
        'hdfs': 'V1alpha1HDFSArtifact',
        'http': 'V1alpha1HTTPArtifact',
        'raw': 'V1alpha1RawArtifact',
        's3': 'V1alpha1S3Artifact'
    }

    attribute_map = {
        'archive_logs': 'archiveLogs',
        'artifactory': 'artifactory',
        'git': 'git',
        'hdfs': 'hdfs',
        'http': 'http',
        'raw': 'raw',
        's3': 's3'
    }

    def __init__(self, archive_logs=None, artifactory=None, git=None, hdfs=None, http=None, raw=None, s3=None):  # noqa: E501
        """V1alpha1ArtifactLocation - a model defined in Swagger"""  # noqa: E501

        self._archive_logs = None
        self._artifactory = None
        self._git = None
        self._hdfs = None
        self._http = None
        self._raw = None
        self._s3 = None
        self.discriminator = None

        if archive_logs is not None:
            self.archive_logs = archive_logs
        if artifactory is not None:
            self.artifactory = artifactory
        if git is not None:
            self.git = git
        if hdfs is not None:
            self.hdfs = hdfs
        if http is not None:
            self.http = http
        if raw is not None:
            self.raw = raw
        if s3 is not None:
            self.s3 = s3

    @property
    def archive_logs(self):
        """Gets the archive_logs of this V1alpha1ArtifactLocation.  # noqa: E501


        :return: The archive_logs of this V1alpha1ArtifactLocation.  # noqa: E501
        :rtype: bool
        """
        return self._archive_logs

    @archive_logs.setter
    def archive_logs(self, archive_logs):
        """Sets the archive_logs of this V1alpha1ArtifactLocation.


        :param archive_logs: The archive_logs of this V1alpha1ArtifactLocation.  # noqa: E501
        :type: bool
        """

        self._archive_logs = archive_logs

    @property
    def artifactory(self):
        """Gets the artifactory of this V1alpha1ArtifactLocation.  # noqa: E501


        :return: The artifactory of this V1alpha1ArtifactLocation.  # noqa: E501
        :rtype: V1alpha1ArtifactoryArtifact
        """
        return self._artifactory

    @artifactory.setter
    def artifactory(self, artifactory):
        """Sets the artifactory of this V1alpha1ArtifactLocation.


        :param artifactory: The artifactory of this V1alpha1ArtifactLocation.  # noqa: E501
        :type: V1alpha1ArtifactoryArtifact
        """

        self._artifactory = artifactory

    @property
    def git(self):
        """Gets the git of this V1alpha1ArtifactLocation.  # noqa: E501


        :return: The git of this V1alpha1ArtifactLocation.  # noqa: E501
        :rtype: V1alpha1GitArtifact
        """
        return self._git

    @git.setter
    def git(self, git):
        """Sets the git of this V1alpha1ArtifactLocation.


        :param git: The git of this V1alpha1ArtifactLocation.  # noqa: E501
        :type: V1alpha1GitArtifact
        """

        self._git = git

    @property
    def hdfs(self):
        """Gets the hdfs of this V1alpha1ArtifactLocation.  # noqa: E501


        :return: The hdfs of this V1alpha1ArtifactLocation.  # noqa: E501
        :rtype: V1alpha1HDFSArtifact
        """
        return self._hdfs

    @hdfs.setter
    def hdfs(self, hdfs):
        """Sets the hdfs of this V1alpha1ArtifactLocation.


        :param hdfs: The hdfs of this V1alpha1ArtifactLocation.  # noqa: E501
        :type: V1alpha1HDFSArtifact
        """

        self._hdfs = hdfs

    @property
    def http(self):
        """Gets the http of this V1alpha1ArtifactLocation.  # noqa: E501


        :return: The http of this V1alpha1ArtifactLocation.  # noqa: E501
        :rtype: V1alpha1HTTPArtifact
        """
        return self._http

    @http.setter
    def http(self, http):
        """Sets the http of this V1alpha1ArtifactLocation.


        :param http: The http of this V1alpha1ArtifactLocation.  # noqa: E501
        :type: V1alpha1HTTPArtifact
        """

        self._http = http

    @property
    def raw(self):
        """Gets the raw of this V1alpha1ArtifactLocation.  # noqa: E501


        :return: The raw of this V1alpha1ArtifactLocation.  # noqa: E501
        :rtype: V1alpha1RawArtifact
        """
        return self._raw

    @raw.setter
    def raw(self, raw):
        """Sets the raw of this V1alpha1ArtifactLocation.


        :param raw: The raw of this V1alpha1ArtifactLocation.  # noqa: E501
        :type: V1alpha1RawArtifact
        """

        self._raw = raw

    @property
    def s3(self):
        """Gets the s3 of this V1alpha1ArtifactLocation.  # noqa: E501


        :return: The s3 of this V1alpha1ArtifactLocation.  # noqa: E501
        :rtype: V1alpha1S3Artifact
        """
        return self._s3

    @s3.setter
    def s3(self, s3):
        """Sets the s3 of this V1alpha1ArtifactLocation.


        :param s3: The s3 of this V1alpha1ArtifactLocation.  # noqa: E501
        :type: V1alpha1S3Artifact
        """

        self._s3 = s3

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(V1alpha1ArtifactLocation, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1alpha1ArtifactLocation):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Argo

    Python client for Argo Workflows  # noqa: E501

    OpenAPI spec version: master
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class V1ListOptions(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'allow_watch_bookmarks': 'bool',
        '_continue': 'str',
        'field_selector': 'str',
        'label_selector': 'str',
        'limit': 'str',
        'resource_version': 'str',
        'timeout_seconds': 'str',
        'watch': 'bool'
    }

    attribute_map = {
        'allow_watch_bookmarks': 'allowWatchBookmarks',
        '_continue': 'continue',
        'field_selector': 'fieldSelector',
        'label_selector': 'labelSelector',
        'limit': 'limit',
        'resource_version': 'resourceVersion',
        'timeout_seconds': 'timeoutSeconds',
        'watch': 'watch'
    }

    def __init__(self, allow_watch_bookmarks=None, _continue=None, field_selector=None, label_selector=None, limit=None, resource_version=None, timeout_seconds=None, watch=None):  # noqa: E501
        """V1ListOptions - a model defined in Swagger"""  # noqa: E501

        self._allow_watch_bookmarks = None
        self.__continue = None
        self._field_selector = None
        self._label_selector = None
        self._limit = None
        self._resource_version = None
        self._timeout_seconds = None
        self._watch = None
        self.discriminator = None

        if allow_watch_bookmarks is not None:
            self.allow_watch_bookmarks = allow_watch_bookmarks
        if _continue is not None:
            self._continue = _continue
        if field_selector is not None:
            self.field_selector = field_selector
        if label_selector is not None:
            self.label_selector = label_selector
        if limit is not None:
            self.limit = limit
        if resource_version is not None:
            self.resource_version = resource_version
        if timeout_seconds is not None:
            self.timeout_seconds = timeout_seconds
        if watch is not None:
            self.watch = watch

    @property
    def allow_watch_bookmarks(self):
        """Gets the allow_watch_bookmarks of this V1ListOptions.  # noqa: E501

        allowWatchBookmarks requests watch events with type \"BOOKMARK\". Servers that do not implement bookmarks may ignore this flag and bookmarks are sent at the server's discretion. Clients should not assume bookmarks are returned at any specific interval, nor may they assume the server will send any BOOKMARK event during a session. If this is not a watch, this field is ignored. If the feature gate WatchBookmarks is not enabled in apiserver, this field is ignored.  This field is beta.  +optional  # noqa: E501

        :return: The allow_watch_bookmarks of this V1ListOptions.  # noqa: E501
        :rtype: bool
        """
        return self._allow_watch_bookmarks

    @allow_watch_bookmarks.setter
    def allow_watch_bookmarks(self, allow_watch_bookmarks):
        """Sets the allow_watch_bookmarks of this V1ListOptions.

        allowWatchBookmarks requests watch events with type \"BOOKMARK\". Servers that do not implement bookmarks may ignore this flag and bookmarks are sent at the server's discretion. Clients should not assume bookmarks are returned at any specific interval, nor may they assume the server will send any BOOKMARK event during a session. If this is not a watch, this field is ignored. If the feature gate WatchBookmarks is not enabled in apiserver, this field is ignored.  This field is beta.  +optional  # noqa: E501

        :param allow_watch_bookmarks: The allow_watch_bookmarks of this V1ListOptions.  # noqa: E501
        :type: bool
        """

        self._allow_watch_bookmarks = allow_watch_bookmarks

    @property
    def _continue(self):
        """Gets the _continue of this V1ListOptions.  # noqa: E501

        The continue option should be set when retrieving more results from the server. Since this value is server defined, clients may only use the continue value from a previous query result with identical query parameters (except for the value of continue) and the server may reject a continue value it does not recognize. If the specified continue value is no longer valid whether due to expiration (generally five to fifteen minutes) or a configuration change on the server, the server will respond with a 410 ResourceExpired error together with a continue token. If the client needs a consistent list, it must restart their list without the continue field. Otherwise, the client may send another list request with the token received with the 410 error, the server will respond with a list starting from the next key, but from the latest snapshot, which is inconsistent from the previous list results - objects that are created, modified, or deleted after the first list request will be included in the response, as long as their keys are after the \"next key\".  This field is not supported when watch is true. Clients may start a watch from the last resourceVersion value returned by the server and not miss any modifications.  # noqa: E501

        :return: The _continue of this V1ListOptions.  # noqa: E501
        :rtype: str
        """
        return self.__continue

    @_continue.setter
    def _continue(self, _continue):
        """Sets the _continue of this V1ListOptions.

        The continue option should be set when retrieving more results from the server. Since this value is server defined, clients may only use the continue value from a previous query result with identical query parameters (except for the value of continue) and the server may reject a continue value it does not recognize. If the specified continue value is no longer valid whether due to expiration (generally five to fifteen minutes) or a configuration change on the server, the server will respond with a 410 ResourceExpired error together with a continue token. If the client needs a consistent list, it must restart their list without the continue field. Otherwise, the client may send another list request with the token received with the 410 error, the server will respond with a list starting from the next key, but from the latest snapshot, which is inconsistent from the previous list results - objects that are created, modified, or deleted after the first list request will be included in the response, as long as their keys are after the \"next key\".  This field is not supported when watch is true. Clients may start a watch from the last resourceVersion value returned by the server and not miss any modifications.  # noqa: E501

        :param _continue: The _continue of this V1ListOptions.  # noqa: E501
        :type: str
        """

        self.__continue = _continue

    @property
    def field_selector(self):
        """Gets the field_selector of this V1ListOptions.  # noqa: E501


        :return: The field_selector of this V1ListOptions.  # noqa: E501
        :rtype: str
        """
        return self._field_selector

    @field_selector.setter
    def field_selector(self, field_selector):
        """Sets the field_selector of this V1ListOptions.


        :param field_selector: The field_selector of this V1ListOptions.  # noqa: E501
        :type: str
        """

        self._field_selector = field_selector

    @property
    def label_selector(self):
        """Gets the label_selector of this V1ListOptions.  # noqa: E501


        :return: The label_selector of this V1ListOptions.  # noqa: E501
        :rtype: str
        """
        return self._label_selector

    @label_selector.setter
    def label_selector(self, label_selector):
        """Sets the label_selector of this V1ListOptions.


        :param label_selector: The label_selector of this V1ListOptions.  # noqa: E501
        :type: str
        """

        self._label_selector = label_selector

    @property
    def limit(self):
        """Gets the limit of this V1ListOptions.  # noqa: E501

        limit is a maximum number of responses to return for a list call. If more items exist, the server will set the `continue` field on the list metadata to a value that can be used with the same initial query to retrieve the next set of results. Setting a limit may return fewer than the requested amount of items (up to zero items) in the event all requested objects are filtered out and clients should only use the presence of the continue field to determine whether more results are available. Servers may choose not to support the limit argument and will return all of the available results. If limit is specified and the continue field is empty, clients may assume that no more results are available. This field is not supported if watch is true.  The server guarantees that the objects returned when using continue will be identical to issuing a single list call without a limit - that is, no objects created, modified, or deleted after the first request is issued will be included in any subsequent continued requests. This is sometimes referred to as a consistent snapshot, and ensures that a client that is using limit to receive smaller chunks of a very large result can ensure they see all possible objects. If objects are updated during a chunked list the version of the object that was present at the time the first list result was calculated is returned.  # noqa: E501

        :return: The limit of this V1ListOptions.  # noqa: E501
        :rtype: str
        """
        return self._limit

    @limit.setter
    def limit(self, limit):
        """Sets the limit of this V1ListOptions.

        limit is a maximum number of responses to return for a list call. If more items exist, the server will set the `continue` field on the list metadata to a value that can be used with the same initial query to retrieve the next set of results. Setting a limit may return fewer than the requested amount of items (up to zero items) in the event all requested objects are filtered out and clients should only use the presence of the continue field to determine whether more results are available. Servers may choose not to support the limit argument and will return all of the available results. If limit is specified and the continue field is empty, clients may assume that no more results are available. This field is not supported if watch is true.  The server guarantees that the objects returned when using continue will be identical to issuing a single list call without a limit - that is, no objects created, modified, or deleted after the first request is issued will be included in any subsequent continued requests. This is sometimes referred to as a consistent snapshot, and ensures that a client that is using limit to receive smaller chunks of a very large result can ensure they see all possible objects. If objects are updated during a chunked list the version of the object that was present at the time the first list result was calculated is returned.  # noqa: E501

        :param limit: The limit of this V1ListOptions.  # noqa: E501
        :type: str
        """

        self._limit = limit

    @property
    def resource_version(self):
        """Gets the resource_version of this V1ListOptions.  # noqa: E501


        :return: The resource_version of this V1ListOptions.  # noqa: E501
        :rtype: str
        """
        return self._resource_version

    @resource_version.setter
    def resource_version(self, resource_version):
        """Sets the resource_version of this V1ListOptions.


        :param resource_version: The resource_version of this V1ListOptions.  # noqa: E501
        :type: str
        """

        self._resource_version = resource_version

    @property
    def timeout_seconds(self):
        """Gets the timeout_seconds of this V1ListOptions.  # noqa: E501


        :return: The timeout_seconds of this V1ListOptions.  # noqa: E501
        :rtype: str
        """
        return self._timeout_seconds

    @timeout_seconds.setter
    def timeout_seconds(self, timeout_seconds):
        """Sets the timeout_seconds of this V1ListOptions.


        :param timeout_seconds: The timeout_seconds of this V1ListOptions.  # noqa: E501
        :type: str
        """

        self._timeout_seconds = timeout_seconds

    @property
    def watch(self):
        """Gets the watch of this V1ListOptions.  # noqa: E501


        :return: The watch of this V1ListOptions.  # noqa: E501
        :rtype: bool
        """
        return self._watch

    @watch.setter
    def watch(self, watch):
        """Sets the watch of this V1ListOptions.


        :param watch: The watch of this V1ListOptions.  # noqa: E501
        :type: bool
        """

        self._watch = watch

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(V1ListOptions, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1ListOptions):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



"""Argo Workflows Python DSL."""

__all__ = [
    # modules
    "task",
    "templates",

    # decorators
    "tasks",
    "template",

    # main
    "Workflow"
]

# modules
from . import tasks
from . import templates

# decorators
from .tasks import task
from .templates import template

# main
from ._workflow import Workflow



# coding: utf-8

"""
    Argo

    Python client for Argo Workflows  # noqa: E501

    OpenAPI spec version: master
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class V1alpha1HDFSKrbConfig(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'krb_c_cache_secret': 'V1SecretKeySelector',
        'krb_config_config_map': 'V1ConfigMapKeySelector',
        'krb_keytab_secret': 'V1SecretKeySelector',
        'krb_realm': 'str',
        'krb_service_principal_name': 'str',
        'krb_username': 'str'
    }

    attribute_map = {
        'krb_c_cache_secret': 'krbCCacheSecret',
        'krb_config_config_map': 'krbConfigConfigMap',
        'krb_keytab_secret': 'krbKeytabSecret',
        'krb_realm': 'krbRealm',
        'krb_service_principal_name': 'krbServicePrincipalName',
        'krb_username': 'krbUsername'
    }

    def __init__(self, krb_c_cache_secret=None, krb_config_config_map=None, krb_keytab_secret=None, krb_realm=None, krb_service_principal_name=None, krb_username=None):  # noqa: E501
        """V1alpha1HDFSKrbConfig - a model defined in Swagger"""  # noqa: E501

        self._krb_c_cache_secret = None
        self._krb_config_config_map = None
        self._krb_keytab_secret = None
        self._krb_realm = None
        self._krb_service_principal_name = None
        self._krb_username = None
        self.discriminator = None

        if krb_c_cache_secret is not None:
            self.krb_c_cache_secret = krb_c_cache_secret
        if krb_config_config_map is not None:
            self.krb_config_config_map = krb_config_config_map
        if krb_keytab_secret is not None:
            self.krb_keytab_secret = krb_keytab_secret
        if krb_realm is not None:
            self.krb_realm = krb_realm
        if krb_service_principal_name is not None:
            self.krb_service_principal_name = krb_service_principal_name
        if krb_username is not None:
            self.krb_username = krb_username

    @property
    def krb_c_cache_secret(self):
        """Gets the krb_c_cache_secret of this V1alpha1HDFSKrbConfig.  # noqa: E501

        KrbCCacheSecret is the secret selector for Kerberos ccache Either ccache or keytab can be set to use Kerberos.  # noqa: E501

        :return: The krb_c_cache_secret of this V1alpha1HDFSKrbConfig.  # noqa: E501
        :rtype: V1SecretKeySelector
        """
        return self._krb_c_cache_secret

    @krb_c_cache_secret.setter
    def krb_c_cache_secret(self, krb_c_cache_secret):
        """Sets the krb_c_cache_secret of this V1alpha1HDFSKrbConfig.

        KrbCCacheSecret is the secret selector for Kerberos ccache Either ccache or keytab can be set to use Kerberos.  # noqa: E501

        :param krb_c_cache_secret: The krb_c_cache_secret of this V1alpha1HDFSKrbConfig.  # noqa: E501
        :type: V1SecretKeySelector
        """

        self._krb_c_cache_secret = krb_c_cache_secret

    @property
    def krb_config_config_map(self):
        """Gets the krb_config_config_map of this V1alpha1HDFSKrbConfig.  # noqa: E501

        KrbConfig is the configmap selector for Kerberos config as string It must be set if either ccache or keytab is used.  # noqa: E501

        :return: The krb_config_config_map of this V1alpha1HDFSKrbConfig.  # noqa: E501
        :rtype: V1ConfigMapKeySelector
        """
        return self._krb_config_config_map

    @krb_config_config_map.setter
    def krb_config_config_map(self, krb_config_config_map):
        """Sets the krb_config_config_map of this V1alpha1HDFSKrbConfig.

        KrbConfig is the configmap selector for Kerberos config as string It must be set if either ccache or keytab is used.  # noqa: E501

        :param krb_config_config_map: The krb_config_config_map of this V1alpha1HDFSKrbConfig.  # noqa: E501
        :type: V1ConfigMapKeySelector
        """

        self._krb_config_config_map = krb_config_config_map

    @property
    def krb_keytab_secret(self):
        """Gets the krb_keytab_secret of this V1alpha1HDFSKrbConfig.  # noqa: E501

        KrbKeytabSecret is the secret selector for Kerberos keytab Either ccache or keytab can be set to use Kerberos.  # noqa: E501

        :return: The krb_keytab_secret of this V1alpha1HDFSKrbConfig.  # noqa: E501
        :rtype: V1SecretKeySelector
        """
        return self._krb_keytab_secret

    @krb_keytab_secret.setter
    def krb_keytab_secret(self, krb_keytab_secret):
        """Sets the krb_keytab_secret of this V1alpha1HDFSKrbConfig.

        KrbKeytabSecret is the secret selector for Kerberos keytab Either ccache or keytab can be set to use Kerberos.  # noqa: E501

        :param krb_keytab_secret: The krb_keytab_secret of this V1alpha1HDFSKrbConfig.  # noqa: E501
        :type: V1SecretKeySelector
        """

        self._krb_keytab_secret = krb_keytab_secret

    @property
    def krb_realm(self):
        """Gets the krb_realm of this V1alpha1HDFSKrbConfig.  # noqa: E501

        KrbRealm is the Kerberos realm used with Kerberos keytab It must be set if keytab is used.  # noqa: E501

        :return: The krb_realm of this V1alpha1HDFSKrbConfig.  # noqa: E501
        :rtype: str
        """
        return self._krb_realm

    @krb_realm.setter
    def krb_realm(self, krb_realm):
        """Sets the krb_realm of this V1alpha1HDFSKrbConfig.

        KrbRealm is the Kerberos realm used with Kerberos keytab It must be set if keytab is used.  # noqa: E501

        :param krb_realm: The krb_realm of this V1alpha1HDFSKrbConfig.  # noqa: E501
        :type: str
        """

        self._krb_realm = krb_realm

    @property
    def krb_service_principal_name(self):
        """Gets the krb_service_principal_name of this V1alpha1HDFSKrbConfig.  # noqa: E501

        KrbServicePrincipalName is the principal name of Kerberos service It must be set if either ccache or keytab is used.  # noqa: E501

        :return: The krb_service_principal_name of this V1alpha1HDFSKrbConfig.  # noqa: E501
        :rtype: str
        """
        return self._krb_service_principal_name

    @krb_service_principal_name.setter
    def krb_service_principal_name(self, krb_service_principal_name):
        """Sets the krb_service_principal_name of this V1alpha1HDFSKrbConfig.

        KrbServicePrincipalName is the principal name of Kerberos service It must be set if either ccache or keytab is used.  # noqa: E501

        :param krb_service_principal_name: The krb_service_principal_name of this V1alpha1HDFSKrbConfig.  # noqa: E501
        :type: str
        """

        self._krb_service_principal_name = krb_service_principal_name

    @property
    def krb_username(self):
        """Gets the krb_username of this V1alpha1HDFSKrbConfig.  # noqa: E501

        KrbUsername is the Kerberos username used with Kerberos keytab It must be set if keytab is used.  # noqa: E501

        :return: The krb_username of this V1alpha1HDFSKrbConfig.  # noqa: E501
        :rtype: str
        """
        return self._krb_username

    @krb_username.setter
    def krb_username(self, krb_username):
        """Sets the krb_username of this V1alpha1HDFSKrbConfig.

        KrbUsername is the Kerberos username used with Kerberos keytab It must be set if keytab is used.  # noqa: E501

        :param krb_username: The krb_username of this V1alpha1HDFSKrbConfig.  # noqa: E501
        :type: str
        """

        self._krb_username = krb_username

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(V1alpha1HDFSKrbConfig, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1alpha1HDFSKrbConfig):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import print_function

from pathlib import Path

from setuptools import setup
from setuptools import find_packages


HERE = Path(__file__).parent

ABOUT = dict()
exec(Path(HERE, "argo/workflows/dsl", "__about__.py").read_text(), ABOUT)

DEVELOPMENT_STATUS = "2 - Pre-Alpha"

README: str = Path(HERE, "README.md").read_text(encoding="utf-8")
REQUIREMENTS: list = Path(HERE, "requirements.txt").read_text().splitlines()


setup_args = dict(
    name=ABOUT["__title__"],
    version=ABOUT["__version__"],
    author=ABOUT["__author__"],
    author_email=ABOUT["__email__"],
    url=ABOUT["__uri__"],
    license=ABOUT["__license__"],
    description=ABOUT["__summary__"],
    long_description=README,
    long_description_content_type="text/markdown",
    classifiers=[
        "Development Status :: %s" % DEVELOPMENT_STATUS,
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Software Development",
        "Topic :: Utilities",
    ],
    packages=["argo.workflows.%s" % p for p in find_packages(where="argo/workflows/")],
    zip_safe=False,
    install_requires=REQUIREMENTS,
)

if __name__ == "__main__":
    setup(**setup_args)



# coding: utf-8

"""
    Argo

    Python client for Argo Workflows  # noqa: E501

    OpenAPI spec version: master
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from argo.workflows.client.api_client import ApiClient


class ArchivedWorkflowServiceApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def delete_archived_workflow(self, uid, **kwargs):  # noqa: E501
        """delete_archived_workflow  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_archived_workflow(uid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str uid: (required)
        :return: object
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.delete_archived_workflow_with_http_info(uid, **kwargs)  # noqa: E501
        else:
            (data) = self.delete_archived_workflow_with_http_info(uid, **kwargs)  # noqa: E501
            return data

    def delete_archived_workflow_with_http_info(self, uid, **kwargs):  # noqa: E501
        """delete_archived_workflow  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_archived_workflow_with_http_info(uid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str uid: (required)
        :return: object
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['uid']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method delete_archived_workflow" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'uid' is set
        if ('uid' not in params or
                params['uid'] is None):
            raise ValueError("Missing the required parameter `uid` when calling `delete_archived_workflow`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'uid' in params:
            path_params['uid'] = params['uid']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/api/v1/archived-workflows/{uid}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='object',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_archived_workflow(self, uid, **kwargs):  # noqa: E501
        """get_archived_workflow  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_archived_workflow(uid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str uid: (required)
        :return: V1alpha1Workflow
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_archived_workflow_with_http_info(uid, **kwargs)  # noqa: E501
        else:
            (data) = self.get_archived_workflow_with_http_info(uid, **kwargs)  # noqa: E501
            return data

    def get_archived_workflow_with_http_info(self, uid, **kwargs):  # noqa: E501
        """get_archived_workflow  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_archived_workflow_with_http_info(uid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str uid: (required)
        :return: V1alpha1Workflow
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['uid']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_archived_workflow" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'uid' is set
        if ('uid' not in params or
                params['uid'] is None):
            raise ValueError("Missing the required parameter `uid` when calling `get_archived_workflow`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'uid' in params:
            path_params['uid'] = params['uid']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/api/v1/archived-workflows/{uid}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='V1alpha1Workflow',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def list_archived_workflows(self, **kwargs):  # noqa: E501
        """list_archived_workflows  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.list_archived_workflows(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str list_options_label_selector: A selector to restrict the list of returned objects by their labels. Defaults to everything. +optional.
        :param str list_options_field_selector: A selector to restrict the list of returned objects by their fields. Defaults to everything. +optional.
        :param bool list_options_watch: Watch for changes to the described resources and return them as a stream of add, update, and remove notifications. Specify resourceVersion. +optional.
        :param bool list_options_allow_watch_bookmarks: allowWatchBookmarks requests watch events with type \"BOOKMARK\". Servers that do not implement bookmarks may ignore this flag and bookmarks are sent at the server's discretion. Clients should not assume bookmarks are returned at any specific interval, nor may they assume the server will send any BOOKMARK event during a session. If this is not a watch, this field is ignored. If the feature gate WatchBookmarks is not enabled in apiserver, this field is ignored.  This field is beta.  +optional
        :param str list_options_resource_version: When specified with a watch call, shows changes that occur after that particular version of a resource. Defaults to changes from the beginning of history. When specified for list: - if unset, then the result is returned from remote storage based on quorum-read flag; - if it's 0, then we simply return what we currently have in cache, no guarantee; - if set to non zero, then the result is at least as fresh as given rv. +optional.
        :param str list_options_timeout_seconds: Timeout for the list/watch call. This limits the duration of the call, regardless of any activity or inactivity. +optional.
        :param str list_options_limit: limit is a maximum number of responses to return for a list call. If more items exist, the server will set the `continue` field on the list metadata to a value that can be used with the same initial query to retrieve the next set of results. Setting a limit may return fewer than the requested amount of items (up to zero items) in the event all requested objects are filtered out and clients should only use the presence of the continue field to determine whether more results are available. Servers may choose not to support the limit argument and will return all of the available results. If limit is specified and the continue field is empty, clients may assume that no more results are available. This field is not supported if watch is true.  The server guarantees that the objects returned when using continue will be identical to issuing a single list call without a limit - that is, no objects created, modified, or deleted after the first request is issued will be included in any subsequent continued requests. This is sometimes referred to as a consistent snapshot, and ensures that a client that is using limit to receive smaller chunks of a very large result can ensure they see all possible objects. If objects are updated during a chunked list the version of the object that was present at the time the first list result was calculated is returned.
        :param str list_options_continue: The continue option should be set when retrieving more results from the server. Since this value is server defined, clients may only use the continue value from a previous query result with identical query parameters (except for the value of continue) and the server may reject a continue value it does not recognize. If the specified continue value is no longer valid whether due to expiration (generally five to fifteen minutes) or a configuration change on the server, the server will respond with a 410 ResourceExpired error together with a continue token. If the client needs a consistent list, it must restart their list without the continue field. Otherwise, the client may send another list request with the token received with the 410 error, the server will respond with a list starting from the next key, but from the latest snapshot, which is inconsistent from the previous list results - objects that are created, modified, or deleted after the first list request will be included in the response, as long as their keys are after the \"next key\".  This field is not supported when watch is true. Clients may start a watch from the last resourceVersion value returned by the server and not miss any modifications.
        :return: V1alpha1WorkflowList
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.list_archived_workflows_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.list_archived_workflows_with_http_info(**kwargs)  # noqa: E501
            return data

    def list_archived_workflows_with_http_info(self, **kwargs):  # noqa: E501
        """list_archived_workflows  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.list_archived_workflows_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str list_options_label_selector: A selector to restrict the list of returned objects by their labels. Defaults to everything. +optional.
        :param str list_options_field_selector: A selector to restrict the list of returned objects by their fields. Defaults to everything. +optional.
        :param bool list_options_watch: Watch for changes to the described resources and return them as a stream of add, update, and remove notifications. Specify resourceVersion. +optional.
        :param bool list_options_allow_watch_bookmarks: allowWatchBookmarks requests watch events with type \"BOOKMARK\". Servers that do not implement bookmarks may ignore this flag and bookmarks are sent at the server's discretion. Clients should not assume bookmarks are returned at any specific interval, nor may they assume the server will send any BOOKMARK event during a session. If this is not a watch, this field is ignored. If the feature gate WatchBookmarks is not enabled in apiserver, this field is ignored.  This field is beta.  +optional
        :param str list_options_resource_version: When specified with a watch call, shows changes that occur after that particular version of a resource. Defaults to changes from the beginning of history. When specified for list: - if unset, then the result is returned from remote storage based on quorum-read flag; - if it's 0, then we simply return what we currently have in cache, no guarantee; - if set to non zero, then the result is at least as fresh as given rv. +optional.
        :param str list_options_timeout_seconds: Timeout for the list/watch call. This limits the duration of the call, regardless of any activity or inactivity. +optional.
        :param str list_options_limit: limit is a maximum number of responses to return for a list call. If more items exist, the server will set the `continue` field on the list metadata to a value that can be used with the same initial query to retrieve the next set of results. Setting a limit may return fewer than the requested amount of items (up to zero items) in the event all requested objects are filtered out and clients should only use the presence of the continue field to determine whether more results are available. Servers may choose not to support the limit argument and will return all of the available results. If limit is specified and the continue field is empty, clients may assume that no more results are available. This field is not supported if watch is true.  The server guarantees that the objects returned when using continue will be identical to issuing a single list call without a limit - that is, no objects created, modified, or deleted after the first request is issued will be included in any subsequent continued requests. This is sometimes referred to as a consistent snapshot, and ensures that a client that is using limit to receive smaller chunks of a very large result can ensure they see all possible objects. If objects are updated during a chunked list the version of the object that was present at the time the first list result was calculated is returned.
        :param str list_options_continue: The continue option should be set when retrieving more results from the server. Since this value is server defined, clients may only use the continue value from a previous query result with identical query parameters (except for the value of continue) and the server may reject a continue value it does not recognize. If the specified continue value is no longer valid whether due to expiration (generally five to fifteen minutes) or a configuration change on the server, the server will respond with a 410 ResourceExpired error together with a continue token. If the client needs a consistent list, it must restart their list without the continue field. Otherwise, the client may send another list request with the token received with the 410 error, the server will respond with a list starting from the next key, but from the latest snapshot, which is inconsistent from the previous list results - objects that are created, modified, or deleted after the first list request will be included in the response, as long as their keys are after the \"next key\".  This field is not supported when watch is true. Clients may start a watch from the last resourceVersion value returned by the server and not miss any modifications.
        :return: V1alpha1WorkflowList
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['list_options_label_selector', 'list_options_field_selector', 'list_options_watch', 'list_options_allow_watch_bookmarks', 'list_options_resource_version', 'list_options_timeout_seconds', 'list_options_limit', 'list_options_continue']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method list_archived_workflows" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'list_options_label_selector' in params:
            query_params.append(('listOptions.labelSelector', params['list_options_label_selector']))  # noqa: E501
        if 'list_options_field_selector' in params:
            query_params.append(('listOptions.fieldSelector', params['list_options_field_selector']))  # noqa: E501
        if 'list_options_watch' in params:
            query_params.append(('listOptions.watch', params['list_options_watch']))  # noqa: E501
        if 'list_options_allow_watch_bookmarks' in params:
            query_params.append(('listOptions.allowWatchBookmarks', params['list_options_allow_watch_bookmarks']))  # noqa: E501
        if 'list_options_resource_version' in params:
            query_params.append(('listOptions.resourceVersion', params['list_options_resource_version']))  # noqa: E501
        if 'list_options_timeout_seconds' in params:
            query_params.append(('listOptions.timeoutSeconds', params['list_options_timeout_seconds']))  # noqa: E501
        if 'list_options_limit' in params:
            query_params.append(('listOptions.limit', params['list_options_limit']))  # noqa: E501
        if 'list_options_continue' in params:
            query_params.append(('listOptions.continue', params['list_options_continue']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/api/v1/archived-workflows', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='V1alpha1WorkflowList',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)
